/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package gui;

import gui.table.TableProgressBar;
import gui.transfer.InfoPanel;
import gui.util.GuiUtil;
import gui.util.logpanel.LogPanel;
import gui.util.tablemodel.ObjectTableModel;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;


import settings.Settings;
import settings.SettingsLoadable;

import model.DLInfo;
import model.DownloadManager;
import model.JIDModel;
import model.transfer.TransferInfo;
import model.transfer.TransferManager;
import model.transfer.TransferObserver;

import static init.JID2.*;
import static gui.util.GuiUtil.*;

public class BasicDownloadPanel extends JPanel implements TransferObserver,SettingsLoadable {

	private static final long serialVersionUID = 1L;

	protected ObjectTableModel<DLInfo> tableModel;
	protected JTable downloadsTable;
	protected JScrollPane scrollDownloadTable;
	protected JSplitPane splitPane;

	protected DownloadManager dm;
	protected InfoPanel infoPanel;
	protected LogPanel logPanel;

	public BasicDownloadPanel(DownloadManager dm) {
		String progress = rm().getLocalizedString("colPercent");
		tableModel = new ObjectTableModel<>(rm()
				.getLocalizedString("colStatus"), rm().getLocalizedString(
				"colFile"), rm().getLocalizedString("colDownloaded"), progress,
				rm().getLocalizedString("colSpeed"), rm().getLocalizedString(
						"colETA"), rm().getLocalizedString("colSize"), rm()
						.getLocalizedString("colURL"));

		downloadsTable = new JTable(tableModel){
			private static final long serialVersionUID = -4051994749608198357L;

			@Override
			public boolean isCellEditable(int row, int column) {
				return false;
			}
		};
		downloadsTable.setBackground(Color.WHITE);
		downloadsTable.setFillsViewportHeight(true);
		downloadsTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

		// downloadsTable.setShowGrid(true);
		TableProgressBar tableProgress = new TableProgressBar();
		downloadsTable.getColumn(progress).setCellRenderer(tableProgress);
		scrollDownloadTable = new JScrollPane(downloadsTable);
		infoPanel = new InfoPanel(rm());

		logPanel = new LogPanel();
		splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				scrollDownloadTable, logPanel);
		this.dm = dm;
		dm.addObserver(this);

		setLayout(new BorderLayout());
		add(splitPane, BorderLayout.CENTER);
		add(infoPanel, BorderLayout.SOUTH);

//		 for (int i=0;i<50;i++)
//		 try {
//			 dm.addDownload(JIDModel.createDLInfo("http://www"+i+".imagebam.com"));
//		 } catch (java.io.IOException e) {
//			 e.printStackTrace();
//		 } catch (util.exception.CancelException e) {
//			e.printStackTrace();
//		}
	}

	public DLInfo getSelectedDLInfo() {
		return tableModel.getValue(downloadsTable.getSelectedRow());
	}

	@Override
	public void updateAddTransfer(TransferInfo info) {
		DLInfo di=(DLInfo) info;
		tableModel.addRow(di, di.getStatus(), di.getFileName(),
				formatBytes(0L), 0, null, null,
				formatBytes(di.getFileSize()), di.getUrl());
		infoPanel.update(dm.getTransferredBytes(), dm.getFileCounter(),
				dm.getNumberOfFiles());
	}

	@Override
	public void updateRemoveTransfer(TransferInfo info) {
		tableModel.removeRowByValue((DLInfo) info);
		infoPanel.update(dm.getTransferredBytes(), dm.getFileCounter(),
				dm.getNumberOfFiles());
	}

	@Override
	public void updateStopTransfer(TransferInfo info) {
		DLInfo di=(DLInfo) info;
		String status = di.getStatus().toString();
		if (di.getError() !=null && di.getError().getMessage() != null)
			status += " ("+rm().getLocalizedString("statusError")+ ": " + di.getError().getMessage()+")";
		tableModel.setData(di, status, di.getFileName(),
				formatBytes(0L), 0, "", "",
				formatBytes(di.getFileSize()), di.getUrl());
	}

	@Override
	public void updateTransferFinished(TransferInfo info) {
		DLInfo di=(DLInfo) info;
		logPanel.addToLog(rm().getLocalizedString("finish") + " "
				+ di.getFileName() + " (" + di.getUrl() + ")");
		long bytes = dm.getDownloadedBytes();
		infoPanel.update(bytes, dm.getFileCounter(),
				dm.getNumberOfFiles());

		tableModel.removeRowByValue(di);
	}


	@Override
	public void updateSpeed(TransferInfo info) {
		DLInfo di=(DLInfo) info;
		int p = 0;
		long bytes = di.getCurrentBytes();
		long speed = di.getSpeed();

		if (di.getFileSize() > 0)
			p = (int) (bytes * 100 / di.getFileSize());
		tableModel.setData(di,
		di.getStatus(), di.getFileName(),
				formatBytes(di.getCurrentBytes()), p,
				formatSpeed(speed),
				formatETA(speed, di.getFileSize(), bytes),
				formatBytes(di.getFileSize()), di.getUrl()

		);
	}

	@Override
	public void updateTotalSpeed(TransferManager<TransferObserver> tm, long speed) {
		infoPanel.updateSpeed(speed);
	}

//	public void updateError(TransferInfo info) {
//		DLInfo di=(DLInfo) info;
//		String status = di.getStatus().toString();
//		if (di.getError() !=null && di.getError().getMessage() != null)
//			status += " ("+rm().getLocalizedString("statusError")+ ": " + di.getError().getMessage()+")";
//		tableModel.setData(di, status, di.getFileName(), formatBytes(0L),
//				0, "", "", formatBytes(di.getFileSize()), di.getUrl());
//	}

	@Override
	public void updateRemoveAll() {
		tableModel.setRowCount(0);
		logPanel.clear();
		infoPanel.update(0L, 0, 0);
		infoPanel.updateSpeed(0L);
	}

	@Override
	public void updateEnqueue(TransferInfo info) {
		DLInfo di=(DLInfo) info;
		int p=0;
		long bytes=di.getCurrentBytes();
		if (di.getFileSize()>0) p=(int)(bytes*100/di.getFileSize());
		tableModel.setData(di, 
				
				   di.getStatus(),
				   di.getFileName(),
				   formatBytes( bytes),
				   p,
				  "",
				 "",
				  formatBytes( di.getFileSize())						 
				  ); 
		
	}

	@Override
	public void loadSettings(Settings s) {
		GuiUtil.setColumnPosAndWidths(downloadsTable, s.getIntArray(SETTING_COL_POS), s.getIntArray(SETTING_COL_WIDTH));
		
		splitPane.setDividerLocation(JIDModel.getSettings().getInt(
				SETTING_LOG_DIVIDER_LOCATION));
		
	}

	@Override
	public void saveSettings(Settings s) {
		s.set(SETTING_COL_POS, GuiUtil.getColumnPositions(downloadsTable));
		s.set(SETTING_COL_WIDTH, GuiUtil.getColumnWidths(downloadsTable));
		
		s.set(SETTING_LOG_DIVIDER_LOCATION,
				splitPane.getDividerLocation());
	}

	@Override
	public void updateFinishedAll() {
		infoPanel.update(dm.getTransferredBytes(), dm.getFileCounter(), dm.getNumberOfFiles());
		infoPanel.updateSpeed(0L);
		System.out.println("done all");
	}

	public void addToLog(String s) {
		logPanel.addToLog(s);
	}

	@Override
	public void updateStopAllTransfers() {
		infoPanel.updateSpeed(0L);
	}
	
}
