/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package gui;

import gui.settings.NewSettingsDialog;
import gui.util.GuiUtil;
import gui.util.buttonhandler.ButtonHandler;
import gui.util.lookandfeel.LAFMenuRestart;

import init.JID2;
import io.downloaders.newdownloader3.RedirectAction;

import javax.swing.JToolBar;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collection;
import java.util.LinkedList;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.SwingWorker;

import net.util.ProxySettings;

import settings.Settings;
import settings.SettingsLoadable;
import util.exception.CancelException;

import model.DLInfo;
import model.DownloadManager;
import model.JIDModel;
import model.transfer.Status;
import model.transfer.TransferInfo;

import static init.JID2.settings;
import static init.JID2.*;

public class DownloaderGUI extends JFrame implements SettingsLoadable {
	private static final long serialVersionUID = 1L;
	private JMenuBar mBar;
	private JMenu mFile,mView,mHelp;
	private LAFMenuRestart mLAndFs;
	private JMenuItem mSettings,mExit,mAbout,mBrowser,mShowToolbar,mUpdate;
	private JRadioButtonMenuItem mToolBar[]=new JRadioButtonMenuItem[3];
	private JToolBar toolbar;
	private JButton buttonLinksFile,buttonEnter,buttonStartAndStopAll,buttonStop,buttonClear;
	private ButtonHandler radioMenuHandler;
	private AbstractAction startStopAction ; 
//	private WebsiteParserModel wpm;
	private NewEnterDialog eld;
	private JPanel buttonPanel;
	private DownloadManager dm;
	private DefaultDownloadPanel downloadPanel;
	private Container contentPane;
	
	public DownloaderGUI(DownloadManager dm) {
		super("JID - Java Image Downloader - version "+JID2.getInstance().getVersion());

		Status.queued.setDispName(rm().getLocalizedString("statusQueued"));
		Status.loading.setDispName(rm().getLocalizedString("statusLoading"));
		Status.finished.setDispName(rm().getLocalizedString("statusFinished"));
		Status.stopped.setDispName(rm().getLocalizedString("statusStopped")); 
		Status.remove.setDispName("removing");
		RedirectAction.setErrFnF(rm().getLocalizedString("errorNotFound"));
		
//		wpm=WebsiteParserModel.getWebsiteParserModel();
		setSize(640, 480);
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				quit();				
			}		
		});
		//dm=(ScheduledDownloadManager) ScheduledDownloadManager.getDownloadManager();
		this.dm=dm;
		contentPane=getContentPane();
		contentPane.setLayout(new BorderLayout());
		downloadPanel= new DefaultDownloadPanel(dm);
		
		contentPane.add(downloadPanel,BorderLayout.CENTER);
		
		buttonLinksFile=new JButton(rm().getLocalizedString("import"));
		buttonEnter=new JButton(rm().getLocalizedString("enter"));
		
		buttonStartAndStopAll=new JButton(rm().getLocalizedString("startStop"));
		buttonStartAndStopAll.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					DownloaderGUI.this.dm.startStopAll();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		});
		
		buttonStop=new JButton(rm().getLocalizedString("stopEnqueue"));
		buttonStop.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				stopEnqueue(downloadPanel.getSelectedDLInfo().getId());
			}
		});
		
		buttonLinksFile.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				importFromFile();
			}
		});
		
		
		buttonClear=new JButton(rm().getLocalizedString("clear"));
		buttonClear.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				clear();
			}
		});
	
		
		buttonPanel=new JPanel();
		buttonPanel.setLayout(new GridLayout(2, 3));
		buttonPanel.add(buttonLinksFile);
		buttonPanel.add(buttonEnter);
		buttonPanel.add(buttonStartAndStopAll);
		buttonPanel.add(buttonStop);
		buttonPanel.add(buttonClear);
		
		mBar=new JMenuBar();
		
		radioMenuHandler=new ButtonHandler();
		mView=new JMenu(rm().getLocalizedString("menuView"));
		mShowToolbar=new JMenu(rm().getLocalizedString("menuToolbar"));
		
		for (int i=0;i<mToolBar.length;i++) {
			final int j=i;
			mToolBar[i]=new JRadioButtonMenuItem(rm().getLocalizedString("menuToolbar"+i));
			mToolBar[i].addActionListener(new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent e) {
					setToolbarOption(j);					
				}
			});
			mShowToolbar.add(mToolBar[i]);
			radioMenuHandler.add(mToolBar[i]);
		}
		
		mView.add(mShowToolbar);
		
		mFile=new JMenu(rm().getLocalizedString("menuFile"));
		mSettings=new JMenuItem(rm().getLocalizedString("menuSettings"));
		mExit=new JMenuItem(rm().getLocalizedString("menuExit"));
		
		mBrowser=new JMenuItem(rm().getLocalizedString("menuBrowser"));
		
		mAbout=new JMenuItem(rm().getLocalizedString("menuAbout"));
		mAbout.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				new AboutDialog().setVisible(true);	
			}
		});
		
		mLAndFs=new LAFMenuRestart(JID2.getInstance(), JID2.SETTING_LOOK_AND_FEEL,rm().getLocalizedString("msgLaFRestart"));
		mView.add(mLAndFs);
		mHelp=new JMenu("?");
		mHelp.add(mAbout);
		
		mBar.add(mFile);
		mBar.add(mView);
		mBar.add(mHelp);
		mFile.add(mSettings);
		mFile.add(mBrowser);
		mSettings.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				NewSettingsDialog sd=new NewSettingsDialog(DownloaderGUI.this);
				sd.setSize(500, 350);
				try {
					Settings s=settings();
					sd.showDialog();
					DownloaderGUI.this.dm.setMaxThreads(s.getInt(SETTING_THREADS));
					DownloaderGUI.this.dm.setProxy(ProxySettings.getProxySettings(s));
				} catch (IOException e1) {
					e1.printStackTrace();
				} catch (CancelException e1) {
					e1.printStackTrace();
				}
			}
		});
		
		mExit.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				quit();		
			}
		});
		
		mBrowser.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
//				if (!settings().getString("browser").equals("")) {
//					try {
//						new ProcessBuilder(settings().getString("browser"),new File(settings().getString("outputDir")).getAbsolutePath()).start();
//					} catch (IOException e1) {
//						
//					}
//				}
				try {
					Path p=JIDModel.getOutput();
					if (!Files.exists(p)) Files.createDirectories(p);
					Desktop.getDesktop().browse(p.toUri());
				} catch (IOException e1) {
					e1.printStackTrace();
				}
			}
		});
		
		mUpdate=new JMenuItem(rm().getLocalizedString("menuUpdate"));
		mUpdate.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				checkForUpdates();
			}
		});
		
		
		toolbar=new JToolBar();
		toolbar.setFloatable(false);
		toolbar.add(new TooltipAbstractAction(rm().getLocalizedString("import"),rm().getIcon("importIcon")) {
			
			private static final long serialVersionUID = 1L;

			@Override
			public void actionPerformed(ActionEvent e) {
				importFromFile();
			}
		});

		toolbar.add(new TooltipAbstractAction(rm().getLocalizedString("enter"),rm().getIcon("enterIcon")) {
			
			private static final long serialVersionUID = 1L;

			@Override
			public void actionPerformed(ActionEvent e) {
				enterLinks();		
			}
		});
		
		startStopAction=new StartStopAction();
		toolbar.add(getToolbarSeparator());
		toolbar.add(startStopAction) ;
		toolbar.add(new TooltipAbstractAction(rm().getLocalizedString("stopEnqueue"),rm().getIcon("enqueueIcon")) {
			
			private static final long serialVersionUID = 1L;

			@Override
			public void actionPerformed(ActionEvent e) {
				if (DownloaderGUI.this.dm.getFileCounter()>0)
					stopEnqueue(downloadPanel.getSelectedDLInfo().getId());		
			}
		});
		toolbar.add(getToolbarSeparator());
		toolbar.add(new TooltipAbstractAction(rm().getLocalizedString("clear"),rm().getIcon("clearIcon")) {
			
			private static final long serialVersionUID = 1L;

			@Override
			public void actionPerformed(ActionEvent e) {
				clear();		
			}
		});
		
		mFile.add(mUpdate);
		mFile.add(mExit);
		
		setJMenuBar(mBar);
		setContentPane(contentPane);
		
		loadSettings(settings());
		downloadPanel.loadSettings(settings());
		/*try {
			List<String> session=JIDModel.loadSession();
			for (String s:session)
				dm.addDownload(JIDModel.createDLInfo(s, null));
			for (int i=0;i<9;i++) {
				DLInfo di=JIDModel.createDLInfo("http://test.down.load");
				di.setFileSize(1024*1024*4+1024*i);
				dm.addDownload(di);
			}
		} catch (IOException e1) {
		}*/
		
		if (settings().getBoolean("updateOnStartup")) checkForUpdates();
		setVisible(true);
	}
	
	private void checkForUpdates() {
		new SwingWorker<Void, Void>() {
			boolean updated;
			@Override
			protected Void doInBackground() throws Exception {
				try {
					downloadPanel.addToLog(rm().getLocalizedString("checkUpdate"));
					updated = JIDModel.downloadUpdates();
				} catch (IOException e1) {
					e1.printStackTrace();
					JOptionPane.showMessageDialog(null, e1.getMessage());
					updated=false;
				}
				return null;
			}
			
			@Override
			protected void done() {
				if (updated)
					downloadPanel.addToLog(rm().getLocalizedString("installationFinished"));
				else
					downloadPanel.addToLog(rm().getLocalizedString("checkNotAvailable"));
			}
		}.execute();
		
	}
	
	private void quit() {
		try {
			dm.stopAll();
		
			saveSettings(JID2.settings());
			JID2.getInstance().quit();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	/*private void save() {
		dm.stopAll();
		downloadPanel.saveSettings();
		try {
			JIDModel.save(dm);
			settings().set("windowState",getExtendedState());
			settings().set("windowX",getX());
			settings().set("windowY",getY());
			settings().set("windowWidth",getWidth());
			settings().set("windowHeight",getHeight());
			settings().set("toolbarOption",radioMenuHandler.getSelectedIndex());
			JID2.getInstance().saveSettings();
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, e.getMessage());
		}
	}*/
	
	
	public void setToolbarOption(int i) {
		if (i==0) {
			contentPane.invalidate();
			contentPane.remove(buttonPanel);
			contentPane.add(toolbar,BorderLayout.NORTH);
			contentPane.validate();
		} else if (i==1) {
			contentPane.invalidate();
			contentPane.remove(toolbar);
			GridLayout gl=((GridLayout)buttonPanel.getLayout());
			gl.setRows(1);
			gl.setColumns(6);
			contentPane.add(buttonPanel,BorderLayout.NORTH);
			contentPane.validate();
		} else if (i==2) {
			contentPane.invalidate();
			contentPane.remove(toolbar);
			GridLayout gl=((GridLayout)buttonPanel.getLayout());
			gl.setRows(2);
			gl.setColumns(3);
			contentPane.add(buttonPanel,BorderLayout.NORTH);
			contentPane.validate();
		}
	}
	
	private class StartStopAction extends TooltipAbstractAction {

		private static final long serialVersionUID = 1L;

		public StartStopAction() {
			super(rm().getLocalizedString("startStop"), rm().getIcon("startStopIcon"));
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			try {
				dm.startStopAll();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}
		
	}
	private abstract class TooltipAbstractAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		public TooltipAbstractAction(String desc,Icon icon) {
			super(desc, icon);
			putValue(javax.swing.Action.SHORT_DESCRIPTION,desc);
		}
	}
	
	private void clear() {
		dm.removeAll();
	}
	
	private void enterLinks() {
		/*try {
			if (eld==null)eld=new EnterLinksDialog(this);
			if (eld.showDialog(wpm.getLinks(Util.getClipboardString()))==EnterLinksPanel.OK) {
	
				for (String l:eld.getLinks()) {
					if (l.trim().length()>0) {
						String subdir=eld.getSubdir();
						if (subdir!=null && subdir.equals("<HOSTER>"))subdir=JIDModel.getHost(l);
						dm.addDownload(JIDModel.createDLInfo(l,subdir ) );
					}
				}
				dm.start(); 
			}
		} catch (IOException e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, e.getMessage());
		}*/
		if (eld==null)eld=new NewEnterDialog(this);
		try {
			Set<String>links= eld.showDialog();
			eld.saveSettings(JID2.settings());
			for (String l:links) {
				if (l.length()>0) {
					String subdir=eld.getSubdir();
					if (subdir!=null && subdir.equals("<HOSTER>"))subdir=JIDModel.getHost(l);
					dm.addDownload(JIDModel.createDLInfo(l,subdir ) );
				}
			}
			
			dm.startAll();
		} catch (CancelException e) {
			System.out.println("cancel");
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, e.getMessage());
		}
	}	
	
	private JToolBar.Separator getToolbarSeparator() {
		Dimension d=new Dimension(16, 32);
		return new JToolBar.Separator(d);
	}
	
	private void importFromFile() {
		JFileChooser fc=new JFileChooser();
		
		if (fc.showOpenDialog(null)==JFileChooser.APPROVE_OPTION) {
			File container=fc.getSelectedFile();
			try {
				BufferedReader br=new BufferedReader(new FileReader(container));
				
				String s=br.readLine();
				
				while (s!=null) {
					dm.addDownload(JIDModel.createDLInfo(s));
					s=br.readLine();
				}
				br.close();
			} catch (IOException e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, e.getMessage());
			} catch (CancelException e) {
				e.printStackTrace();
			}
		}
	}
	
	/*private void extractFromWebsite() {
		if (wp==null) wp= new WebsiteExtractDialog(this,wpm);
		if (wp.showDialog()==EnterLinksPanel.OK) {
			try {
				for (String l: wp.getLinks()) {
					DLInfo di=JIDModel.createDLInfo(l, wp.getSubdir());
					dm.addDownload(di);
				}
			} catch (IOException e) {
			}
			dm.start();
		}
	}*/
	
	private void stopEnqueue(int id) {
		try {
			dm.stopEnqueue(id);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private class DefaultDownloadPanel extends BasicDownloadPanel {

		private static final long serialVersionUID = 1L;
		private JPopupMenu popup=new JPopupMenu();
		
		public DefaultDownloadPanel(DownloadManager dm) {
			super(dm);
			
			JMenuItem mnuCopy =new JMenuItem(rm().getLocalizedString("copy"));
			mnuCopy.addActionListener(new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent e) {
					int[] rows=downloadsTable.getSelectedRows();
					LinkedList<String> list=new LinkedList<String>();
					int col=tableModel.findColumn("URL");
					for (int row:rows) {
						list.add(tableModel.getValueAt(row, col).toString());
					}
					GuiUtil.setClipboardString(list);
				}
			});
			
			JMenuItem mnuRemove=new JMenuItem(rm().getLocalizedString("removeSelected"));
			mnuRemove.addActionListener(new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent e) {
					int[] rows=downloadsTable.getSelectedRows();
					DLInfo[] toRemove=new DLInfo[rows.length];
					for (int i=0;i<rows.length;i++) {
						toRemove[i]=tableModel.getValue(rows[i]);
					}
					for (int i=0;i<rows.length;i++) {
						DownloaderGUI.this.dm.remove(toRemove[i].getId());
					}
				}
			});
			
			JMenuItem mnuRemoveErr=new JMenuItem(rm().getLocalizedString("removeNonexistent"));
			mnuRemoveErr.addActionListener(new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent e) {
					Collection<TransferInfo > dls=DownloaderGUI.this.dm.getUnfinished();
					
					for (TransferInfo di:dls) {
						Exception ex=di.getError();
						if (ex instanceof FileNotFoundException)  DownloaderGUI.this.dm.remove(di.getId());
					}
				}
			});
			
//			JMenuItem mnuEditUrl=new JMenuItem(rm().getLocalizedString("editURL"));
//			mnuEditUrl.addActionListener(new ActionListener() {
//				
//				@Override
//				public void actionPerformed(ActionEvent e) {
//									
//					DLInfo di=tableModel.getValue(downloadsTable.getSelectedRow());
//					
//					if (di.getStatus()==Status.stopped ||di.getStatus()==Status.error) {
//						String input=JOptionPane.showInputDialog(rm().getLocalizedString("newURL"),di.getUrl());
//						
//						if (input!=null && input.startsWith("http://")) {
//							DownloaderGUI.this.dm.changeUrl(di, input);
//						}
//					}
//				}
//			}); 
			
			
			popup.add(mnuRemove);
			popup.add(mnuRemoveErr);
//			popup.add(mnuEditUrl);
			popup.add(mnuCopy);
			downloadsTable.addMouseListener(new MouseListener() {
				
				@Override
				public void mouseReleased(MouseEvent e) {
					if (e.isPopupTrigger()) {
						popup.show(e.getComponent(), e.getX(), e.getY());
					}
				}
				
				@Override
				public void mousePressed(MouseEvent e) {
					if (e.isPopupTrigger()) {
						popup.show(e.getComponent(), e.getX(), e.getY());
					}				
				}
				
				@Override
				public void mouseExited(MouseEvent e) {}
				
				@Override
				public void mouseEntered(MouseEvent e) {}
				
				@Override
				public void mouseClicked(MouseEvent e) {}
			});
		}
		
	}

	@Override
	public void loadSettings(Settings s) {
		setExtendedState(s.getInt(SETTING_WINDOW_STATE));
		setBounds(s.getInt(SETTING_X),s.getInt(SETTING_Y),s.getInt(SETTING_WIDTH),s.getInt(SETTING_HEIGHT) );
		int tbo=settings().getInt(SETTING_TOOLBAR);
		radioMenuHandler.setSelectedButton(tbo);
		setToolbarOption(tbo);
		try {
			JID2.getInstance().loadSession(dm);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (CancelException e) {
			e.printStackTrace();
		}
	}


	@Override
	public void saveSettings(Settings s) {
		s.set(SETTING_X, getX());
		s.set(SETTING_Y, getY());
		s.set(SETTING_WIDTH, getWidth());
		s.set(SETTING_HEIGHT,getHeight());
		s.set(SETTING_WINDOW_STATE, getExtendedState());
		s.set(SETTING_TOOLBAR,radioMenuHandler.getSelectedIndex());
		downloadPanel.saveSettings(s);
		try {
			JID2.getInstance().saveSession(dm);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	
	
	
}
