/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package gui;

import static init.JID2.*;

import gui.layout.AnchorLayout;
import gui.layout.AnchorParam;
import gui.layout.PercentageLayout;
import gui.okcancelpanel.OKCancelPanel;
import gui.util.GuiUtil;

import init.JID2;
import io.IDownloaderConfig;
import io.downloaders.HTMLDownloader;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.text.Document;
import javax.swing.undo.UndoManager;

import model.DLInfo;
import model.JIDModel;
import model.WebsiteParserModel;
import model.util.Util;

import settings.Settings;
import settings.SettingsLoadable;
import util.exception.CancelException;


public class NewEnterDialog extends JDialog implements SettingsLoadable{

	private static final long serialVersionUID = 3843709838211753246L;
	
	private OKCancelPanel okCancel;
	private JPopupMenu popup;
	private JTextArea txtLinks;
	private JMenuItem undo,cut,copy,paste,selectAll,clear,mFilter;
	private WebsiteParserModel wpm;
	private SubdirPanel pOutput;
	private JCheckBox chkMonitor,chkAutoLoad,chkOnTop;
	private JButton cmdLoad,cmdPaste,cmdFromClipboard;
	private JTextField txtUrl;
	private String lastClipboardString;
	private Timer monitorTimer;
//	private Set<String> links;
//	private JCheckBox chkAllLinks;
//	private JTextField txtAttribute;
	
	public NewEnterDialog(Window owner ) {
		super(owner,rm().getLocalizedString("enter"));
		setModal(true);
		txtLinks=new JTextArea(){
			private static final long serialVersionUID = -7097132381659084507L;

			@Override
			public void paste() {
				String[] list= GuiUtil.getClipboardString().split("\n");
				addLinks(list);
			}
		};
		
		final UndoManager manager = new UndoManager();
		final Document document = txtLinks.getDocument();
		document.addUndoableEditListener(manager);
		
		okCancel=new OKCancelPanel(this, rm().getLocalizedString("ok"),rm().getLocalizedString("cancel"));
		okCancel.setPreferredSize(new Dimension(getWidth(), 40));
		
		popup=new JPopupMenu();
		txtLinks.add(popup);
		cut=new JMenuItem(rm().getLocalizedString("cut"));
		copy=new JMenuItem(rm().getLocalizedString("copy"));
		paste=new JMenuItem(rm().getLocalizedString("paste"));
		
		undo=new JMenuItem(rm().getLocalizedString("undo"));
		selectAll=new JMenuItem(rm().getLocalizedString("selectAll"));
		clear=new JMenuItem(rm().getLocalizedString("clear"));
		mFilter=new JMenuItem("Filter");
		mFilter.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				String[] links = txtLinks.getText().split("\n");
				String expr=JOptionPane.showInputDialog(rm().getLocalizedString("msgFilter"));
				if (expr!=null) {
					StringBuilder sb=new StringBuilder();
					for (int i=0;i<links.length;i++) {
						if (links[i].contains(expr)){
							sb.append(links[i]);
							sb.append('\n');
						}
					}
					txtLinks.setText(sb.toString());
				}
			}
		});
		
		popup.add(undo);
		popup.addSeparator();
		popup.add(cut);
		popup.add(copy);
		popup.add(paste);
		popup.addSeparator();
		popup.add(mFilter);
		popup.addSeparator();
		popup.add(selectAll);
		popup.add(clear);
		
		cut.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				txtLinks.cut();
				popup.setVisible(false);
			}
		});
		
		copy.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				
				txtLinks.copy();
				popup.setVisible(false);
			}
		});
		
		paste.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				txtLinks.paste();
				popup.setVisible(false);
			}
		});
		
		selectAll.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				txtLinks.selectAll();
				popup.setVisible(false);
			}
		}); 
		
		undo.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				if (manager.canUndo())	manager.undo();	
			}
		});
		
		clear.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				clear();				
			}
		});
		
		txtLinks.addMouseListener(new MouseAdapter() {
			
			@Override
			public void mouseReleased(MouseEvent e) {
				if (e.isPopupTrigger()) {
					showPopup(e);
				}
			}
			
			@Override
			public void mousePressed(MouseEvent e) {
				if (e.isPopupTrigger()) {
					showPopup(e);
				}
			}
						
			private void showPopup(MouseEvent e) {
				popup.show(e.getComponent(), e.getX(), e.getY());
			}
		});
		
		JPanel pOption=new JPanel(new PercentageLayout());
		JPanel pUrl=new JPanel(new AnchorLayout());
		pOutput=new SubdirPanel();
		/*txtAttribute = new JTextField();
		JPanel pAdvanced = new JPanel(new AnchorPercentLayout());
		chkAllLinks = new JCheckBox("Retrieve all links");
		pAdvanced.add(chkAllLinks, APConstraint.constraint().left(5).rightPercent(50).top(10));
		pAdvanced.add(new JLabel("Attribute"), APConstraint.constraint().left(5).rightPercent(50));
		pAdvanced.add(txtAttribute, APConstraint.constraint().leftPercent(50).right(5));
		pAdvanced.setBorder(BorderFactory.createTitledBorder("Advanced"));*/
		pUrl.setBorder(BorderFactory.createTitledBorder("URL"));
		pOption.add(pUrl,PercentageLayout.constraints(0, 0, 100, 50));
		pOption.add(pOutput,PercentageLayout.constraints(0, 50, 100, 50));
//		pOption.add(pAdvanced,PercentageLayout.constraints(50, 50, 50, 50));
		pOption.setPreferredSize(new Dimension(getWidth(), 220));
		Container cp=getContentPane();
		((JComponent) cp).setBorder(BorderFactory.createEmptyBorder(10, 10, 10,10));
		cp.setLayout(new BorderLayout());
		cp.add(pOption,BorderLayout.NORTH);
		cp.add(new JScrollPane(txtLinks),BorderLayout.CENTER);
		
		chkOnTop=new JCheckBox(rm().getLocalizedString("onTop"));
		chkOnTop.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				setAlwaysOnTop(chkOnTop.isSelected());				
			}
		});
		JPanel p=new JPanel(new AnchorLayout());
		p.add(okCancel,AnchorLayout.getConstraintsByBorder(0, 0, 0, 0));
		p.add(chkOnTop,AnchorLayout.getConstraints(AnchorParam.right(10),AnchorParam.height(24)));
		
		
		fillUrlPanel(pUrl);
		
		cmdFromClipboard=new JButton(rm().getLocalizedString("getLinks"));
		p.add(cmdFromClipboard, AnchorLayout.getConstraints(
				AnchorParam.left(10),
				AnchorParam.height(30)
				));
		cmdFromClipboard.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				new SwingWorker<Void, Void>() {
					@Override
					protected Void doInBackground() throws Exception {
						cmdFromClipboard.setEnabled(false);
						paste();	
						return null;
					}
					
					@Override
					protected void done() {
						cmdFromClipboard.setEnabled(true);
					}
				}.execute();
							
			}
		});
		
		
		p.setComponentZOrder(chkOnTop, 0);
		p.setComponentZOrder(cmdFromClipboard, 0);
		p.setPreferredSize(okCancel.getPreferredSize());
		cp.add(p,BorderLayout.SOUTH);
		
		wpm=WebsiteParserModel.getWebsiteParserModel();
		loadSettings(JID2.settings());
	}
	
	
	private void fillUrlPanel(JPanel pUrl) {
		txtUrl=new JTextField();
		cmdLoad=new JButton( rm().getLocalizedString("loadSite"));
		cmdPaste=new JButton(rm().getLocalizedString("paste"));
		chkMonitor=new JCheckBox(rm().getLocalizedString("monitorClipboard"),JIDModel.getSettings().getBoolean("monitorClipboard"));
		chkAutoLoad=new  JCheckBox(rm().getLocalizedString("autoLoad"),JIDModel.getSettings().getBoolean("monitorClipboard"));
		chkAutoLoad.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				autoload();
			}
		});
		
		chkMonitor.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				monitor();
			}
		});
		cmdLoad.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				if (txtUrl.getText().isEmpty()) getUrlFromClipboard();
				parse();
							
			}
		}); 
		
		cmdPaste.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				getUrlFromClipboard();				
			}
		});
		pUrl.add(txtUrl,AnchorLayout.getConstraints(AnchorParam.left(20),AnchorParam.top(5),AnchorParam.right(220),AnchorParam.height(24)));
		pUrl.add(cmdPaste,AnchorLayout.getConstraints(AnchorParam.right(120),AnchorParam.width(100),AnchorParam.height(30),AnchorParam.top(0)));
		pUrl.add(cmdLoad,AnchorLayout.getConstraints(AnchorParam.right(20),AnchorParam.width(100),AnchorParam.height(30),AnchorParam.top(0)));
		pUrl.add(chkMonitor,AnchorLayout.getConstraints(AnchorParam.left(20),AnchorParam.bottom(5),AnchorParam.height(24)));
		pUrl.add(chkAutoLoad,AnchorLayout.getConstraints(AnchorParam.right(20),AnchorParam.bottom(5),AnchorParam.height(24)));
	}
	
	private void autoload() {
		if (chkAutoLoad.isSelected()) {
			parse();
		}
	}
	
	private void parse() {
		new Thread(new Runnable() {

			@Override
			public void run() {
				cmdLoad.setText(rm().getLocalizedString("loading"));
				cmdLoad.setEnabled(false);
				try {
					String urlString=txtUrl.getText();

					if (urlString.isEmpty()) {
						cmdLoad.setEnabled(true);
						cmdLoad.setText(rm().getLocalizedString("loadSite"));
						return;
					}
					
					if (!urlString.startsWith(Util.HTTP_TOKEN)) {
						urlString=Util.HTTP_TOKEN+urlString;
					}
					
					final DLInfo dlInfo=JIDModel.createDLInfo(urlString,null);
					HTMLDownloader downloader =new HTMLDownloader(dlInfo);
										
					downloader.startTransfer();
					
					/*Set<String > links= chkAllLinks.isSelected() 
							? wpm.getLinks(downloader.getResponse(),txtUrl.getText(),txtAttribute.getText(),null)
									: wpm.getLinks(downloader.getResponse(),txtUrl.getText());*/
					addLinks(wpm.getLinks(downloader.getResponse(),txtUrl.getText()));
					
					cmdLoad.setEnabled(true);
					cmdLoad.setText(rm().getLocalizedString("loadSite"));
				} catch (IOException ioex) {
					ioex.printStackTrace();
					//JOptionPane.showMessageDialog(NewEnterDialog.this, ioex.getMessage(), rm().getLocalizedString("errors"), JOptionPane.ERROR_MESSAGE);
				} catch (CancelException e) {
					e.printStackTrace();
				}	
				
			}

		}).start();
		
	}
	
//	private void addLink(String l) {
//		prepareAddLinks();
//		txtLinks.append(l);
//		txtLinks.append("\n");
//	}

	private void addLinks(Set<String> links) {
		prepareAddLinks();
		
		for (String l:links){
			txtLinks.append(l);
			txtLinks.append("\n");
		}
	}
	private void addLink(String link) {
		prepareAddLinks();
		txtLinks.append(link);
		txtLinks.append("\n");
	}
	private void addLinks(String[] links) {
		prepareAddLinks();
		
		for (String l:links){
			txtLinks.append(l);
			txtLinks.append("\n");
		}
	}

	private void prepareAddLinks() {
		if (!txtLinks.getText().endsWith("\n")&&txtLinks.getText().length()>0) {
			txtLinks.append("\n");
		}
	}
	
	private void monitor() {
		if (chkMonitor.isSelected()) {
			
			monitorTimer=new Timer();
			monitorTimer.schedule(new TimerTask() {
				
				@Override
				public void run() {
					if (lastClipboardString==null) {
						lastClipboardString=GuiUtil.getClipboardString();
						if (lastClipboardString!=null) {
							if (chkAutoLoad.isSelected()) {
								parse();
							} else {
								getUrlFromClipboard();
							}
						}
						
					} else {
						String s=GuiUtil.getClipboardString();
						
						if (s!=null) {
							if (!s.equals(lastClipboardString)) {
								lastClipboardString=s;
								getUrlFromClipboard();
								autoload();
							}
						}
					}
				}
			}, 0,300);		
		} else {
			if (monitorTimer!=null) {
				monitorTimer.cancel();
				lastClipboardString=null;
			}
		}
	}
	private void paste() {
		
		try {
			addLinks(wpm.getLinks(GuiUtil.getClipboardString()));
		} catch (IOException e) {
			e.printStackTrace();
		} catch (CancelException e) {
			e.printStackTrace();
		}
	}

	private void clear() {
		txtLinks.setText("");
	}
	
	
	private void getUrlFromClipboard() {
		String s=GuiUtil.getClipboardString();
		System.out.println(s);
		
		if (s!=null) {
			if (!s.startsWith(Util.HTTP_TOKEN)) s=Util.HTTP_TOKEN+s;
			
			if (!Util.urlContainsIllegalChar(s) && s.contains(".")) {
				try{
				IDownloaderConfig cfg= JIDModel.getDownloaderConfigByUrl(s);
					if ( cfg==null ) {
						txtUrl.setText(s);
					} else {
						txtUrl.setText("");
						addLink(s);
					}
				}catch (Exception e) {
				}
			}
		}
	}
	
	public void updateUITree() {
		SwingUtilities.updateComponentTreeUI(getContentPane());
		SwingUtilities.updateComponentTreeUI(popup);
	}
	
	public Set<String> showDialog() throws CancelException {
		clear();
		paste();
		setVisible(true);
		okCancel.getRetVal();
		Set<String> links=new LinkedHashSet<>();
		String s=txtLinks.getText();
		
		int start=0,end=s.indexOf('\n');
		
		while ( (end=s.indexOf('\n',start))>-1) {
			String l=s.substring(start, end).trim();
			if (l.length()>0)
				links.add(l);
			start=end+1;
		}
		String l=s.substring(start).trim();
		if (l.length()>0)
			links.add(l);
		
		if (monitorTimer!=null)
			monitorTimer.cancel();
		dispose();
		return links;
	}

	public String getSubdir() {
		return pOutput.getSubdir();
	}

	@Override
	public void loadSettings(Settings s) {
		setBounds(s.getInt(SETTING_ENTER_LINKS_WINDOW_X),
		s.getInt(SETTING_ENTER_LINKS_WINDOW_Y),
		s.getInt(SETTING_ENTER_LINKS_WINDOW_WIDTH),
		s.getInt(SETTING_ENTER_LINKS_WINDOW_HEIGHT));
		chkMonitor.setSelected(s.getBoolean(SETTING_URL_CB));
		chkAutoLoad.setSelected(s.getBoolean(SETTING_AUTO_LOAD_URL));
		pOutput.setCustomSubdir(s.getString(SETTING_CUSTOM_SUBDIR_ENTER));
		pOutput.setSelectedSubdirOption(s.getInt(SETTING_SUBDIR_OPTION_ENTER) );
		chkOnTop.setSelected(s.getBoolean(SETTING_ENTER_WINDOW_ON_TOP));
		setAlwaysOnTop(chkOnTop.isSelected());
		paste();
		monitor();
		autoload();		
	}

	@Override
	public void saveSettings(Settings s) {
		s.set(SETTING_SUBDIR_OPTION_ENTER,pOutput.getSelectedSubdirOption());
		s.set(SETTING_CUSTOM_SUBDIR_ENTER,pOutput.getCustomSubdir());
		s.set(SETTING_ENTER_LINKS_WINDOW_X,getX());
		s.set(SETTING_ENTER_LINKS_WINDOW_Y,getY());
		s.set(SETTING_ENTER_LINKS_WINDOW_WIDTH,getWidth());
		s.set(SETTING_ENTER_LINKS_WINDOW_HEIGHT,getHeight());
		s.set(SETTING_URL_CB,chkMonitor.isSelected());
		s.set(SETTING_AUTO_LOAD_URL,chkAutoLoad.isSelected());
		s.set(SETTING_ENTER_WINDOW_ON_TOP,chkOnTop.isSelected());
	}
}
