/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package io;

import net.util.LinkExtractor;
import init.JID2;
import io.downloaders.newdownloader3.AppendToUrlAction;
import io.downloaders.newdownloader3.ConditionalRedirectAction;
import io.downloaders.newdownloader3.DisableContentTypeCheckAction;
import io.downloaders.newdownloader3.DisableGzipAction;
import io.downloaders.newdownloader3.RedirectAction;
import io.downloaders.newdownloader3.ReplaceInURLAction;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import plugin.PluginLoader;


import util.exception.CancelException;
import xml.reader.DefaultXMLReader;

import model.transfer.ConfigUpdate;
import model.urlconfig.URLConfig;
import model.urlconfig.URLContainsConfig;
import model.urlconfig.URLEndsWithConfig;
import model.urlconfig.URLMatchesConfig;
import model.urlconfig.URLNotContainsConfig;
import model.urlconfig.URLStartWithConfig;

public class ConfigLoader3 {
	private static Path cfgDir,hostsFile;
	private static PluginLoader pluginLoader = new PluginLoader();
	
	public static void setCfgDir(Path cfgDir) {
		ConfigLoader3.cfgDir = cfgDir;
		hostsFile =cfgDir.resolve("hosts.xml");
	}
	
	public static Map<String, IDownloaderConfig> load() throws IOException, CancelException {
		try {
			CfgLoaderHandler cl=new CfgLoaderHandler();
			
			if (!Files.exists(hostsFile)) {
				downloadUpdates(true);
			}
			DefaultXMLReader.read(hostsFile, cl );
			return cl.getCfgs();
		} catch (SAXException e) {
			throw new IOException();
		}
	}

	private static URLConfig getUrlConfig(String method,String expr) {
		if (method.equals("matches")) {
			return new URLMatchesConfig(expr);
		} else if (method.equals("contains")) {
			return new URLContainsConfig(expr);
		} else if (method.equals("startsWith")) {
			return new URLStartWithConfig(expr);
		} else if (method.equals("endsWith")) {
			return new URLEndsWithConfig(expr);
		} else if (method.equals("notContains")) {
			return new URLNotContainsConfig(expr);
		} else {
			throw new IllegalArgumentException();
		}
	}


	public static String[] loadDelimiters() throws IOException {
		
		Path f=cfgDir.resolve("parser").resolve("delimiters.cfg");
		if (Files.exists(f)) {
			List<String>d= Files.readAllLines(f,JID2.UTF8);
			
			String[] delimiters=new String[d.size()];
		
			int i=0;
			for (String s:d) {
				
				if (s.equals("\\u0020")) {
					delimiters[i]="\u0020";
				} else if (s.equals("\\n")) {
					delimiters[i]=("\n");
				} else if (s.equals("\\r")) {
					delimiters[i]=("\r");
				} else { 
					delimiters[i]=s;
				}
				i++;
			}
			return delimiters;
		} else {
			throw new FileNotFoundException();
		}
		
	}

	public static Map<String, IDownloaderConfig> loadCustom() throws IOException {
		CfgLoaderHandler2 handler = new CfgLoaderHandler2();
		try {
			DefaultXMLReader.read(cfgDir.resolve("parser").resolve("direct.xml"), handler);
			return handler.getCfgs();
		} catch (SAXException e) {
			throw new IOException();
		}
	}
	
	private static class CfgLoaderHandler implements ContentHandler {
		
		protected Map<String, IDownloaderConfig> cfgs;
		protected IDownloaderConfig currentCfg;
		
		public CfgLoaderHandler() {
			cfgs=new HashMap<String, IDownloaderConfig>();
		}
		
		@Override
		public void setDocumentLocator(Locator locator) {
		}
		
		@Override
		public void startDocument() throws SAXException {
		}
		
		@Override
		public void endDocument() throws SAXException {
		}
		
		@Override
		public void startPrefixMapping(String prefix, String uri) throws SAXException {
		}
		
		@Override
		public void endPrefixMapping(String prefix) throws SAXException {
		}
		
		@Override
		public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
			if (localName.equals("host")){
				String a=atts.getValue("additional");
				boolean additional=a!=null&& a.equals("true");
				String strDirect=atts.getValue("direct");
				IDownloaderConfig cfg;
				boolean direct=strDirect!=null&&strDirect.equals("true");
				if (!additional){
					cfg=new DefaultDownloaderConfig3();
				} else if (direct){
					cfg = new DirectDownloadCfg();
					cfg.addUrlConfig(new URLContainsConfig(atts.getValue("arg")));
				} else {
					Path f=JID2.getInstance().getConfigurationPath().resolve("additional").resolve(atts.getValue("module"));
					cfg=(IDownloaderConfig) pluginLoader.load(f, atts.getValue("class"));
				}
				cfgs.put(atts.getValue("name"), cfg);
				currentCfg = cfg;
					
			} else if (localName.equals("url")) {
				if (atts.getLength() == 2) {
					URLConfig uc=getUrlConfig(
						atts.getValue("method"),
						atts.getValue("arg")
					);
					if (currentCfg == null)
						throw new RuntimeException();
					currentCfg.addUrlConfig(uc);
				}
			} else if (localName.equals("redirect")) {
				String type=atts.getValue("extractorType");
				if (type==null) throw new SAXException();
				LinkExtractor extractor = LinkExtractor.getLinkExtractor(type);
				
				String[] argNames = extractor.getArgNames();
				String[] extractorArgs = new String[argNames.length];
				
				for (int i=0;i<argNames.length;i++) {
					extractorArgs[i] = atts.getValue(argNames[i]);
				}
				
				RedirectAction a=new RedirectAction(extractor, extractorArgs);
				currentCfg.addAction(a);
			} else if (localName.equals("appendToUrl")) {
				currentCfg.addAction( new AppendToUrlAction(atts.getValue("arg") )); 
			} else if (localName.equals("conditionalRedirect")) {
				String type=atts.getValue("extractorType");
				if (type==null) throw new SAXException();
				LinkExtractor extractor = LinkExtractor.getLinkExtractor(type);
				
				String[] argNames = extractor.getArgNames();
				String[] extractorArgs = new String[argNames.length];
				
				for (int i=0;i<argNames.length;i++) {
					extractorArgs[i] = atts.getValue(argNames[i]);
				}
				
				String[] extractorArgsElse = new String[argNames.length];
				
				for (int i=0;i<argNames.length;i++) {
					extractorArgsElse[i] = atts.getValue(argNames[i]+"Else");
				}
				
				ConditionalRedirectAction a=new ConditionalRedirectAction(extractor, atts.getValue("contains"), extractorArgs, extractorArgsElse);
				currentCfg.addAction(a);
			} else if (localName.equals("replaceInUrl")) {
				currentCfg.addAction(new ReplaceInURLAction(atts.getValue("target"), atts.getValue("replace")));
			} else if (localName.equals("disableGzip")) {
				currentCfg.addAction(new DisableGzipAction());
			} else if (localName.equals("disableContentTypeCheck")) {
				currentCfg.addAction(new DisableContentTypeCheckAction());
			}
		}
		
		@Override
		public void endElement(String uri, String localName, String qName) throws SAXException {
		}
		
		@Override
		public void characters(char[] ch, int start, int length) throws SAXException {
		}
		
		@Override
		public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
		}
		
		@Override
		public void processingInstruction(String target, String data) throws SAXException {
		}
		
		@Override
		public void skippedEntity(String name) throws SAXException {
		}
		
		public Map<String, IDownloaderConfig> getCfgs() {
			return cfgs;
		}
	}
	
	private static class CfgLoaderHandler2 extends CfgLoaderHandler {
		public CfgLoaderHandler2() {
			// TODO Auto-generated constructor stub
		}
		
		@Override
		public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
			if (localName.equals("entry")){
				String host = atts.getValue("host");
				DirectDownloadCfg cfg = new DirectDownloadCfg();
				cfg.addUrlConfig(new URLContainsConfig(atts.getValue("arg")));
				cfgs.put(host, cfg);
			}
		}
	}
	
	public static boolean downloadUpdates() throws IOException, CancelException{
		return downloadUpdates(false);
	}
	
	public static boolean downloadUpdates(boolean force) throws IOException, CancelException{
		String type = "additional";
		String name = "jid";
		Path additionalDir=cfgDir.resolve(type);
		
		boolean b1= ConfigUpdate.downloadUpdates(name, "hosts", JID2.getInstance(), cfgDir, force);
		boolean b2= ConfigUpdate.downloadNonXMLUpdates(name, type, JID2.getInstance(),additionalDir , force);
		
		return b1 || b2;		
	}
}
