/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package io.downloaders;

import shoshone.httpclient.ShoshoneClient;
import util.exception.CancelException;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.Files;

import model.DLInfo;

public class FileDownloader extends Downloader {

	private ShoshoneClient urlConn;
	
	public static final char[] ILLEGAL_CHARS= {'<','>','"'};
	
	public FileDownloader(DLInfo dlInfo) throws IOException {
		super(dlInfo, null);
		urlConn=new ShoshoneClient(null,true);
	}

	@Override
	public void startTransfer() throws IOException, CancelException {
		
		if (containsIllegalChars(dlInfo.getUrl())) {
			throw new FileNotFoundException();
		} else {
			urlConn.setProxy(proxy);
			urlConn.setUrl(dlInfo.getUrl());	
			urlConn.setAutoReferer();
			urlConn.setFollowRedirects(true);
			if (Files.exists(dlInfo.getOutputFile())) {
				urlConn.resumeDownload(dlInfo.getOutputFile());
			}
			
			urlConn.applyGetRequest(false);
			
			if (urlConn.getContentLength()!=-1L) dlInfo.setFileSize(urlConn.getContentLength());
			if (dlInfo.getFileName()==null) dlInfo.setFileName(urlConn.getFilename());
			
			createOutDirs();
			urlConn.readResponseAndSave(dlInfo.getOutputFile());

			
			urlConn.closeConnection();
		}
	}

	public String getResponse() throws IOException {
		return urlConn.getResponse();
	}
	
	@Override
	public void updateCurrentBytes() {
		dlInfo.updateBytes(urlConn.getTransferedBytes());
	}

	private boolean containsIllegalChars(String s) {
		for (char c:ILLEGAL_CHARS) {
			if (s.contains(String.valueOf(c))) return true;
		}
		return false;
	}
	
	@Override
	public void cancel() {
		urlConn.cancel();
	}
}
