/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package io.downloaders.newdownloader3;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Iterator;


import shoshone.httpclient.ShoshoneClient;
import util.exception.CancelException;
import model.DLInfo;
import model.transfer.EngineConfig;
import model.urlconfig.URLConfig;
import io.DefaultDownloaderConfig3;
import io.downloaders.Downloader;
import io.downloaders.action.Action;

public class NewDownloader3 extends Downloader {

	protected ShoshoneClient client; 
	
	public NewDownloader3(DLInfo info, EngineConfig cfg) throws IOException {
		super(info, cfg);
		client=new ShoshoneClient(null,true);		
	}

	@Override
	public void updateCurrentBytes() {
		dlInfo.updateBytes(client.getTransferedBytes()	);		
	}

	@Override
	public void startTransfer() throws IOException, CancelException {
		client.setProxy(proxy);
		DefaultDownloaderConfig3 dcfg=(DefaultDownloaderConfig3)cfg;
		
		String u=dlInfo.getUrl();
		if (cfg==null) {
			throw new IOException();
		} else {
			boolean urlMatched=dcfg.getUrlConfigs().size()==0;
			
			for (URLConfig urlcfg:dcfg.getUrlConfigs()) {
				if (urlcfg.matchesExpression(u)) {
					urlMatched=true;
					break;
				}
			}
			
			client.setFollowRedirects(true);
			client.setAutoReferer();
			client.setUrl(dlInfo.getUrl());
			
			if (urlMatched) {
				Iterator<Action> it= dcfg.getActions();
				
				while (it.hasNext()) {
					it.next().apply(client, dlInfo);
				}
				
			}
			if (dlInfo.getOutputFile() != null&& Files.exists(dlInfo.getOutputFile())) {
				client.resumeDownload(dlInfo.getOutputFile());
			}
			client.applyGetRequest(false);
			if (client.getContentLength()!=-1L && dlInfo.getFileSize()==-1) dlInfo.setFileSize(client.getContentLength());
			if (dlInfo.getFileName()==null)	dlInfo.setFileName(client.getFilename());
			
			createOutDirs();
			Path f=dlInfo.getOutputFile();
			client.readResponseAndSave(dlInfo.getOutputFile());
			client.closeConnection();
			if (!dlInfo.typeCheckDisabled() && (client.getResponseCode()>=400 || ShoshoneClient.TYPE_HTML.equals(client.getFullContentType()))) {
				System.out.println(toString());
//				Files.move(f, java.nio.file.Paths.get("/tmp/"+f.getFileName()),java.nio.file.StandardCopyOption.REPLACE_EXISTING);
				throw new IOException();
			}
			if (dlInfo.getFileSize()==-1) dlInfo.setFileSize(Files.size(f));
		}
	}

	@Override
	public void cancel() {
		client.cancel();
	}
	
	@Override
	public String toString() {
		return client.getFullContentType() + " "+ client.getResponseCode();
	}

}
