package jbiu.gui;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.IOException;
import java.util.Collection;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

import core.io.Application;

import static jbiu.init.JBIU.rm;
import jbiu.init.JBIU;
import jbiu.io.account.AccountManager;
import jbiu.model.LoginInfo;
import settings.Settings;
import gui.layout.percent2.APConstraint;
import gui.layout.percent2.AnchorPercentLayout;
import gui.resource.ResourceManager;
import gui.settings.dialog.component.CheckboxSettingsComponent;
import gui.settings.dialog.group.SettingsComponentGroup;
import gui.settings.dialog.page.SettingsPage;
import gui.util.GuiUtil;


public class AccountManagerPanel extends SettingsPage{
	private static final long serialVersionUID = -6584425566596506012L;
	private String lastHosterName=null;
	private JTextField txtUser;
	private JPasswordField txtPass;
	private JLabel lblUser,lblPass;
	private CheckboxSettingsComponent chkUser,chkPass;
	private JComboBox<GUIUploaderConfig> cbHosters;
	private DefaultComboBoxModel<GUIUploaderConfig> cbHostersModel;
	
	public AccountManagerPanel(String entry,Collection<GUIUploaderConfig> cfgs) {
		super(entry, JBIU.getInstance());
		cbHostersModel=new DefaultComboBoxModel<>();
		cbHosters=new JComboBox<>(cbHostersModel);
		for (GUIUploaderConfig cfg:cfgs) {
			if (cfg.getUploaderConfig()!=null) {
				AccountManager.loadRawData(cfg.getUploaderConfig().getInternalName());
				LoginInfo li=cfg.getUploaderConfig().getLoginInfo();
				if (li!=null)
					cbHostersModel.addElement(cfg);
			}
		}
		
		cbHosters.addItemListener(new ItemListener() {
			
			@Override
			public void itemStateChanged(ItemEvent e) {
				int i=cbHosters.getSelectedIndex();
				
				if (e.getStateChange()==ItemEvent.SELECTED && i>-1) {
					checkUserPassChanged();
					
					displayLoginInfo(cbHostersModel.getElementAt(i));
				}
			}
		});		
		txtUser=new JTextField();
		txtPass=new JPasswordField();
		lblUser=new JLabel(rm().getLocalizedString("user"));
		lblPass=new JLabel(rm().getLocalizedString("pass"));
		
		JPanel pnLogin=new JPanel();
		pnLogin.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(rm().getLocalizedString("settingTitleLoginData")),GuiUtil.DEFAULT_EMPTY_BORDER));
		pnLogin.setLayout(new AnchorPercentLayout());
		pnLogin.add(cbHosters,APConstraint.constraint().left(10).right(10).topPercent(5));
		pnLogin.add(lblUser,APConstraint.constraint().left(10).right(10).topPercent(45));
		pnLogin.add(txtUser,APConstraint.constraint().leftPercent(30).right(10).topPercent(45));
		pnLogin.add(lblPass,APConstraint.constraint().left(10).right(10).topPercent(75));
		pnLogin.add(txtPass,APConstraint.constraint().leftPercent(30).right(10).topPercent(75));
		add(pnLogin,APConstraint.linearV(65, 35));
		
		if (cbHostersModel.getSize() > 0)
			displayLoginInfo(cbHostersModel.getElementAt(0));
	}

	private void checkUserPassChanged() {
		if (lastHosterName!=null) {
			String usr=null;
			byte[] pass=null;
			if (chkUser.isSelected())  usr=txtUser.getText();
			if (chkPass.isSelected())  pass=new String(txtPass.getPassword()).getBytes(Application.UTF8);
			try {
				AccountManager.setAndSaveLogin(lastHosterName, usr,pass);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	@Override
	protected void createComponentGroups(Settings settings, ResourceManager rm) {
		setLayout(new AnchorPercentLayout());
		SettingsComponentGroup g=new SettingsComponentGroup(rm.getLocalizedString("saveLoginData"), rm) {
			
			private static final long serialVersionUID = -7479817569396109615L;

			@Override
			protected void createSettingsComponents(final ResourceManager rm) {
				setLayout(new AnchorPercentLayout());
				chkUser=new CheckboxSettingsComponent("saveUser",rm.getLocalizedString("user"));
				chkPass=new CheckboxSettingsComponent("savePass",rm.getLocalizedString("pass"));
				((JCheckBox) chkPass.getComponent()).addActionListener(new ActionListener() {

					@Override
					public void actionPerformed(ActionEvent e) {
						if (chkPass.isSelected())
							JOptionPane.showMessageDialog(null, rm.getLocalizedString("warningSavePass"),rm.getLocalizedString("warning"),JOptionPane.WARNING_MESSAGE);
					}
					
				});
				addSettingsComponent(chkUser,APConstraint.constraint().leftPercent(20));
				addSettingsComponent(chkPass,APConstraint.constraint().rightPercent(20));
			}
		};
		
		addSettingsComponentGroup(g,APConstraint.linearV(30,0));
		
	}

	@Override
	public void save() {
		super.save();
		checkUserPassChanged();
	}
	
	private void displayLoginInfo(GUIUploaderConfig cfg) {
		lastHosterName=cfg.getUploaderConfig().getInternalName();
		lblUser.setText(cfg.getUserDescription());
		lblPass.setText(cfg.getPassDescription());
		
		byte[] pass=AccountManager.getPassword(cfg.getUploaderConfig().getInternalName());
		if (pass!=null) {
			txtPass.setText(new String( pass));
		} else {
			txtPass.setText("");
		}
		
		String usr=AccountManager.getUser(cfg.getUploaderConfig().getInternalName());
		if (usr!=null) {
			txtUser.setText(usr);
		} else {
			txtUser.setText("");
		}
	}
}
