package jbiu.gui;

import static jbiu.init.JBIU.rm;
import static gui.util.GuiUtil.*;

import gui.table.TableProgressBar;
import gui.transfer.InfoPanel;
import gui.util.logpanel.LogPanel;
import gui.util.tablemodel.ObjectTableModel;

import java.awt.BorderLayout;
import java.awt.Color;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;

import settings.Settings;
import settings.SettingsLoadable;
import model.transfer.TransferInfo;
import model.transfer.TransferManager;
import model.transfer.TransferObserver;

import jbiu.model.JBIUModel;
import jbiu.model.ULInfo;
import jbiu.model.UploadManager;

public class BasicUploadPanel extends JPanel implements TransferObserver, SettingsLoadable{
	
	private static final long serialVersionUID = 1L;
	
	protected ObjectTableModel<ULInfo> tableModel;
	protected JTable downloadsTable;
	protected JScrollPane scrollDownloadTable;
	protected JSplitPane splitPane;
	
	protected UploadManager uploadMan;
	protected InfoPanel infoPanel;
	protected LogPanel logPanel;
	
	public BasicUploadPanel(UploadManager um) {
		String progress=rm().getLocalizedString("colPercent");
		tableModel=new ObjectTableModel<ULInfo>(rm().getLocalizedString("colStatus"),
									rm().getLocalizedString("colFile"),
									rm().getLocalizedString("colUploaded"),
									progress,
									rm().getLocalizedString("colSpeed"),
									rm().getLocalizedString("colETA"),
									rm().getLocalizedString("colSize")
									){

			
			private static final long serialVersionUID = -5436550274263850704L;

			@Override
			public boolean isCellEditable(int row, int column) {
				return false;
			}
		};

		downloadsTable=new JTable(tableModel);
		downloadsTable.setBackground(Color.WHITE);
		downloadsTable.setFillsViewportHeight(true);
		downloadsTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		TableProgressBar tableProgress= new TableProgressBar();
		tableProgress.setBackground(Color.WHITE);
		downloadsTable.getColumn(progress).setCellRenderer(tableProgress);
		scrollDownloadTable = new JScrollPane(downloadsTable);
		infoPanel=new InfoPanel(rm());
		
		logPanel=new LogPanel();
		splitPane=new JSplitPane(JSplitPane.VERTICAL_SPLIT, scrollDownloadTable, logPanel);
		this.uploadMan=um;
		um.addObserver(this);
		splitPane.setDividerLocation(JBIUModel.getSettings().getInt("tableLogDividerLocation"));
		setLayout(new BorderLayout());
		add(splitPane,BorderLayout.CENTER  );
		add(infoPanel,BorderLayout.SOUTH);	
	}
	
	public void onFinish() {
	}

	@Override
	public void loadSettings(Settings s) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void saveSettings(Settings s) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void updateAddTransfer(TransferInfo info) {
		ULInfo ulInfo=(ULInfo) info;
		tableModel.addRow(ulInfo,
				  ulInfo.getStatus(),
				  ulInfo.getFileName(),
				  formatBytes(0L),
				  0,
				  null,
				  null,
				  formatBytes( ulInfo.getFileSize())
				  );

		infoPanel.update(uploadMan.getTransferredBytes(), uploadMan.getFileCounter(), uploadMan.getNumberOfFiles());
	}

	@Override
	public void updateRemoveTransfer(TransferInfo info) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void updateStopTransfer(TransferInfo info) {
		ULInfo ulInfo=(ULInfo) info;
		tableModel.setData(ulInfo,
				  ulInfo.getStatus(),
				  ulInfo.getFileName(),
				  formatBytes(0L),
				  0,
				  "",
				  "",
				  formatBytes( ulInfo.getFileSize())
				  );
	}

	@Override
	public void updateTransferFinished(TransferInfo info) {
		ULInfo ulInfo=(ULInfo) info;
		System.out.println("remove");
		tableModel.removeRowByValue(ulInfo);
		logPanel.addToLog(rm().getLocalizedString("finish")+" "+ulInfo.getFileName() );
		long bytes=uploadMan.getTransferredBytes();
		infoPanel.update(bytes, uploadMan.getFileCounter(), uploadMan.getNumberOfFiles());
	}

	@Override
	public void updateSpeed(TransferInfo info) {
		//System.out.println("update speed");
		ULInfo ulInfo=(ULInfo) info;
		int p=0;
		long bytes=ulInfo.getCurrentBytes();
		long speed=ulInfo.getSpeed();
		
		System.out.println(bytes);
		
		if (ulInfo.getFileSize()>0) p=(int)(bytes*100/ulInfo.getFileSize());
		tableModel.setData(ulInfo, 
				
				   ulInfo.getStatus(),
				   ulInfo.getFileName(),
				   formatBytes( bytes),
				   p,
				  formatSpeed(speed),
				  formatETA(speed, ulInfo.getFileSize(), bytes),
				  formatBytes( ulInfo.getFileSize())						 
				  ); 
	}

	@Override
	public void updateTotalSpeed(TransferManager<TransferObserver> tm, long speed) {
		infoPanel.updateSpeed(speed);
	}


	@Override
	public void updateRemoveAll() {
		// TODO Auto-generated method stub
		
	}
	
	/*@Override
	public void update(Observable o, Object arg) {
		try {
		if (arg == null) {
			infoPanel.update(uploadMan.getTransferredBytes(), uploadMan.getTotalSpeed(), uploadMan.getFileCounter(), uploadMan.getNumberOfFiles());
		} else if (arg instanceof ULInfo) {
			ULInfo ulInfo=(ULInfo)arg;
			
			switch (ulInfo.getStatus()) {
			case newTransfer:
				
					tableModel.addRow(ulInfo.getId(),
									  ulInfo.getStatus(),
									  ulInfo.getFileName(),
									  formatBytes(0L),
									  0,
									  null,
									  null,
									  formatBytes( ulInfo.getFileSize())
									  );
				
				infoPanel.update(0,0, uploadMan.getFileCounter(), uploadMan.getNumberOfFiles());
				break;
			case stopped:
				tableModel.setData(ulInfo.getId(),
								  ulInfo.getStatus(),
								  ulInfo.getFileName(),
								  formatBytes(0L),
								  0,
								  "",
								  "",
								  formatBytes( ulInfo.getFileSize())
								  );
				break;
			case finished:
				tableModel.removeRowById(ulInfo.getId());
				logPanel.addToLog(rm().getLocalizedString("finish")+" "+ulInfo.getFileName() );
				long bytes=uploadMan.getTransferredBytes();
				long speed=uploadMan.getTotalSpeed();
				infoPanel.update(bytes, speed, uploadMan.getFileCounter(), uploadMan.getNumberOfFiles());
				
				if (uploadMan.getFileCounter()==uploadMan.getNumberOfFiles()) {
					onFinish();
				}
				break;
			case error:
				String err=ulInfo.getStatus().toString();
				if (ulInfo.getError()!=null && ulInfo.getError().getMessage()!=null) err+=": "+ulInfo.getError().getMessage();
				tableModel.setData(ulInfo.getId(),
						  err,
						  ulInfo.getFileName(),
						  formatBytes(0L),
						  0,
						  "",
						  "",
						  formatBytes( ulInfo.getFileSize())
						  );
				break;
			case remove:
				tableModel.removeRowById(ulInfo.getId());
				break;
			default:
				int p=0;
				bytes=ulInfo.getCurrentBytes();
				speed=ulInfo.getSpeed();
				
				if (ulInfo.getFileSize()>0) p=(int)(bytes*100/ulInfo.getFileSize());
				tableModel.setData(ulInfo.getId(), 
						
						   ulInfo.getStatus(),
						   ulInfo.getFileName(),
						   formatBytes( ulInfo.getCurrentBytes()),
						   p,
						  formatSpeed(speed),
						  formatETA(speed, ulInfo.getFileSize(), bytes),
						  formatBytes( ulInfo.getFileSize())						 
						  ); 
				break;	
			}
		
		}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	
	private void loadSettings() {
		int[] colPos=JBIUModel.getSettings().getIntArray("columnPos");
		for (int i=0;i<colPos.length;i++) {
			
			int p=colPos[i];
			
			int k=downloadsTable.getColumnModel().getColumnIndex(tableModel.getColumnName(p));
			if (k!=i ) {
				downloadsTable.getColumnModel().moveColumn(k,i);
			}

		}
		int[] colWidth=JBIUModel.getSettings().getIntArray("columnWidth");
		if (colWidth!=null ) {
			for (int i=0;i<colWidth.length;i++) {
			
				int w=colWidth[i];
				if (w>0) {
					downloadsTable.getColumnModel().getColumn(i).setPreferredWidth(w);
				}
			}
		
		}
		splitPane.setDividerLocation(JBIUModel.getSettings().getInt("tableLogDividerLocation"));
	}

	
	
	public void saveSettings() {
		int[] columnWidth=new int[downloadsTable.getColumnCount()];
		
		for (int i=0;i<columnWidth.length;i++) {
			columnWidth[i]=downloadsTable.getColumnModel().getColumn(i).getPreferredWidth();
		}
		JBIUModel.getSettings().set("columnWidth", columnWidth);
		int[] columnPos=new int[downloadsTable.getColumnCount()];

		for (int i=0;i<downloadsTable.getColumnCount();i++) {
			columnPos[i]=tableModel.findColumn(downloadsTable.getColumnModel().getColumn(i).getHeaderValue().toString());
		}
		JBIUModel.getSettings().set("columnPos",columnPos);
		JBIUModel.getSettings().set("tableLogDividerLocation",splitPane.getDividerLocation());
	}
	
	public void downloadUpdates() {
		logPanel.addToLog(rm().getLocalizedString("checkUpdate"));
		new Thread(new Runnable() {
			
			@Override
			public void run() {
				try {
					ConfigModel.checkForUpdates(BasicUploadPanel.this);
				} catch (CancellationException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
					logPanel.addToLog("error while downloading");
				}
			}
		}).start();
	}*/
	
	public int getSelectedDownloadId() {
		return tableModel.getValue(downloadsTable.getSelectedRow()).getId(); 
	}

	@Override
	public void updateEnqueue(TransferInfo info) {
		ULInfo ulInfo=(ULInfo) info;
		int p=0;
		long bytes=ulInfo.getCurrentBytes();
		if (ulInfo.getFileSize()>0) p=(int)(bytes*100/ulInfo.getFileSize());
		tableModel.setData(ulInfo, 
				
				   ulInfo.getStatus(),
				   ulInfo.getFileName(),
				   formatBytes( bytes),
				   p,
				  "",
				 "",
				  formatBytes( ulInfo.getFileSize())						 
				  ); 
	}

	@Override
	public void updateFinishedAll() {
		infoPanel.update(uploadMan.getTransferredBytes(), uploadMan.getFileCounter(),uploadMan.getNumberOfFiles());
		infoPanel.updateSpeed(0L);
		onFinish();
	}


	@Override
	public void updateStopAllTransfers() {
		infoPanel.updateSpeed(0L);
	}
}
