package jbiu.gui;

import gui.filelist.FileList;
import gui.filetree.FileTree;
import gui.util.GuiUtil;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.nio.file.Path;
import java.util.List;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;

import settings.Settings;
import settings.SettingsLoadable;
import util.exception.CancelException;

import jbiu.init.JBIU;
import jbiu.model.JBIUModel;
import jbiu.model.ULInfo;
import jbiu.model.UploadManager;
import jbiu.model.UploaderConfig;

import static jbiu.init.JBIU.rm;

public class FileChooserPanel extends JPanel implements SettingsLoadable{

	private static final long serialVersionUID = 1L;

	private FileTree dirChooser;
	private FileList fileList;
	
	private JButton buttonUpload;
	private DefaultComboBoxModel<GUIUploaderConfig> cbModel;
	private JComboBox<GUIUploaderConfig> cbHost;
	private JPanel optionPanel;	
	private UploadManager uploadManager;
	
	public FileChooserPanel(final UploaderGUI gui,UploadManager um) {
		this.uploadManager=um;
		fileList = new FileList(true,rm().getLocalizedString("fileListName"),rm().getLocalizedString("fileListSize"),rm().getLocalizedString("fileListDate"));
		dirChooser = new FileTree(fileList);
		
		buttonUpload=new JButton("Upload");
		buttonUpload.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
//				new SwingWorker<Void, Void>() {
//
//					@Override
//					protected Void doInBackground() throws Exception {
						
						Path[] files=fileList.getSelectedFiles();
						
						if (files!=null) {
							try {
								
								GUIUploaderConfig gcfg=GuiUtil.getSelectedValue(cbHost);
								
								UploaderConfig cfg=gcfg.getUploaderConfig();
								
								boolean useDefTokens=JBIUModel.getSettings().getBoolean("useDefaultTokens");
							
								
								if (useDefTokens) {
									ULInfo[] ulInfos = new ULInfo[files.length];
									for (int i=0;i<ulInfos.length;i++){
										ulInfos[i] =uploadManager.createULInfo(files[i],cfg.getLinkTypeCount());
										ulInfos[i].setFileTokens(cfg.getDefaultFileTokens());
										uploadManager.addUpload(ulInfos[i], cfg);
									}
								} else if (gcfg.getInputComponentCount()>0||gcfg.hasGallery()){
									ULInfo[] ulInfos = new ULInfo[files.length];
									for (int i=0;i<ulInfos.length;i++){
										ulInfos[i] =uploadManager.createULInfo(files[i],cfg.getLinkTypeCount());
									}
									
									new PrepareUploadGUI(gui).display(gcfg, ulInfos);
									
									for (int i=0;i<ulInfos.length;i++){
										uploadManager.addUpload(ulInfos[i], cfg);
									}
								} else {
									for (int i=0;i<files.length;i++){
										uploadManager.addUpload(uploadManager.createULInfo(files[i],cfg.getLinkTypeCount()), cfg);
									}
								}
								
								gui.selectUploadTab();
								
								uploadManager.startAll();
							} catch (CancelException ce) {
								ce.printStackTrace();
							} catch (IOException ioex) {
								ioex.printStackTrace();
								JOptionPane.showMessageDialog(null, ioex.getLocalizedMessage());
							}
						}
					
//						return null;
//					}
//				}.execute();
//					
//				
			}
		});
		
		JSplitPane split1 = new JSplitPane(JSplitPane.VERTICAL_SPLIT, fileList, buttonUpload);
		JSplitPane split2 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, dirChooser, split1);
		
		cbModel=new DefaultComboBoxModel<>();
		cbHost=new JComboBox<>(cbModel);
		
		List<GUIUploaderConfig> cfgs= GUIConfigLoader2.getConfigs();
		cbModel.addElement(GUIConfigLoader2.getSimulator());
		for (GUIUploaderConfig c:cfgs) {
			cbModel.addElement(c);
		}
		
		
		cbHost.setSelectedIndex(1);
		
		optionPanel=new JPanel();
		optionPanel.setLayout(new BorderLayout());
//		optionPanel.add(getHostPanel(cbModel.getSelectedValue()),BorderLayout.CENTER);
		optionPanel.validate();
		JPanel topPanel=new JPanel();
		topPanel.setLayout(new BorderLayout());
		
		Dimension d=new Dimension(640, 50);
		topPanel.setMinimumSize(d);
		topPanel.setPreferredSize(d);
		
		JPanel hostPanel=new JPanel();
		hostPanel.setLayout(new GridLayout(2, 1));
		hostPanel.add(new JLabel("Host"));
		hostPanel.add(cbHost);
		topPanel.add(hostPanel,BorderLayout.WEST);
		topPanel.add(optionPanel,BorderLayout.CENTER);
		setLayout(new BorderLayout());
		add(split2,BorderLayout.CENTER);
		add(topPanel,BorderLayout.NORTH);
		
		split1.setDividerLocation(200);
		split2.setDividerLocation(200);
		
	}
	
//	private JPanel getHostPanel(GUIUploaderConfig cfg ) {
//		Iterator<InputComponent> it=cfg.getInputComponentsIterator();
//		JPanel p=new JPanel();
//		p.setLayout(new GridLayout(1, cfg.getInputComponentCount()));
//		
//		while (it.hasNext()) {
//			p.add(it.next());
//		}
//		return p;
//			
//	}
	

	@Override
	public void loadSettings(Settings s) {
		Path p=s.getPath(JBIU.SETTING_LAST_PATH);
		if (p!=null)
			dirChooser.expandFileTree(p);
	}

	@Override
	public void saveSettings(Settings s) {
		if (dirChooser.getSelectedDir()!=null)
			s.set(JBIU.SETTING_LAST_PATH,dirChooser.getSelectedDir().toString());
	}
}
