package jbiu.gui;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import jbiu.gui.gallery.ExistingGalleryOption;
import jbiu.gui.gallery.GalleryOption;
import jbiu.gui.gallery.NewGalleryOption;
import jbiu.gui.input.model.ComboBoxInputComponentConfig;
import jbiu.gui.input.model.GalleryInputComponentConfig;
import jbiu.gui.input.model.InputComponentConfig;
import jbiu.gui.input.model.TextBoxInputComponentConfig;
import jbiu.init.JBIU;
import jbiu.model.ConfigModel;
import jbiu.model.UploaderConfig;

import model.transfer.ConfigUpdate;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import util.exception.CancelException;
import xml.reader.DefaultXMLReader;

public class GUIConfigLoader2 {

	private static List<GUIUploaderConfig> configs;
	private static GUIUploaderConfig simulator;
	static{
		configs=new ArrayList<>();
		reset();
		simulator=new GUIUploaderConfig(null) {
			@Override
			public UploaderConfig getUploaderConfig() {
				return ConfigModel.getSimulator();
			}
		};
		simulator.setDisplay("-- simulator --");
		simulator.addLinkTypeName("view");
		for (int i=0;i<3;i++)
			simulator.addLinkTypeName("other"+i);
	}
	
	private static void reset(){
		configs.clear();
		try {
			load(JBIU.getInstance().getConfigurationPath());
			Collections.sort(configs);
		} catch (IOException e) {
			throw new RuntimeException("Error loading gui.xml", e);
		} catch (CancelException e) {
			e.printStackTrace();
		}
	}
	
	public static List<GUIUploaderConfig> getConfigs() {
		return configs;
	}
	
	public static boolean downloadUpdates(boolean force) throws IOException, CancelException {
		boolean b= ConfigUpdate.downloadUpdates("jbiu","gui",JBIU.getInstance(), JBIU.getInstance().getConfigurationPath(),force);
		if (b) reset();
		return b;
	}
	
	public static GUIUploaderConfig getNoneSelected(String display){
		GUIUploaderConfig cfg= new GUIUploaderConfig(null){
			@Override
			public UploaderConfig getUploaderConfig() {
				return null;
			}
		};
		cfg.setDisplay(display);
		return cfg;
	}
	
	
	public static GUIUploaderConfig getSimulator(){
		return simulator;
	}
	
	private static void load(Path configurationPath) throws IOException, CancelException{
		/*Path path=configurationPath.resolve("gui");
		LinkedList<GUIUploaderConfig> cfgs = new LinkedList<>();
		DirectoryStream<Path> files=null;
		try{
			files=Files.newDirectoryStream(path,
				new DirectoryStream.Filter<Path>() {
	
					@Override
					public boolean accept(Path entry) throws IOException {
						return entry.getFileName().toString().endsWith(".cfg");
					}
					
				}
			);
			
			for (Path f:files) {
				String name=f.getFileName().toString();
				CfgLoaderHandler cl=new CfgLoaderHandler(name.substring(0, name.length()-4));
				try {
					DefaultXMLReader.read(f.toFile(),cl );
					cfgs.add(cl.cfg);
				} catch (SAXException e) {
					throw new IOException(e);
				}
			}
			
			
			Collections.sort(cfgs);
			configs=cfgs;
		} finally {
			files.close();
		}*/
		
		Path f=configurationPath.resolve("gui.xml");
		if (!Files.exists(f)) {
			downloadUpdates(true);
		}
		CfgLoaderHandler cl=new CfgLoaderHandler();
		try {
			DefaultXMLReader.read(f, cl );
		} catch (SAXException e) {
			throw new IOException(e);
		}
	}

	private static class CfgLoaderHandler implements ContentHandler {

		private InputComponentConfig inputCfg;
		private GalleryOption currGalOpt;
		private GUIUploaderConfig cfg;
		
		@Override
		public void setDocumentLocator(Locator locator) {
			
			
		}

		@Override
		public void startDocument() throws SAXException {
			
			
		}

		@Override
		public void endDocument() throws SAXException {
			
			
		}

		@Override
		public void startPrefixMapping(String prefix, String uri) throws SAXException {
						
		}

		@Override
		public void endPrefixMapping(String prefix) throws SAXException {
			
			
		}

		@Override
		public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
			
			if (localName.equals("host")) {
				cfg=new GUIUploaderConfig(atts.getValue("name"));
				cfg.setDisplay(atts.getValue("display"));
			} else if (cfg != null){
			
				if (localName.equals("link")) {
				
					cfg.addLinkTypeName(atts.getValue("description"));
				} else if (localName.equals("input")) {
					inputCfg=getInputComponentCfg(atts);
					if (!(inputCfg instanceof GalleryInputComponentConfig))
						cfg.add(inputCfg);
				} else if (localName.equals("nogallery")) {
					currGalOpt=((GalleryInputComponentConfig) inputCfg).createNoGalleryOption();
				} else if (localName.equals("newgallery")) {
					currGalOpt=((GalleryInputComponentConfig) inputCfg).createNewGalleryOption();
				} else if (localName.equals("existinggallery")) {
					currGalOpt = null;
				} else if (localName.equals("gallerytoken")) {
					String token=atts.getValue("token");
					String value=atts.getValue("value");
					if (currGalOpt != null) {
						currGalOpt.setToken(token, value);
					} else {
						ExistingGalleryOption.putAdditionalToken(token, value);
					}
				} else if (localName.equals("galleryinput")) {
					if (currGalOpt !=null && currGalOpt instanceof NewGalleryOption) {
						inputCfg=getInputComponentCfg(atts);
						NewGalleryOption ngo= (NewGalleryOption)currGalOpt;
						ngo.addInputComponent(atts.getValue("text"),inputCfg);
					}
				} else if (localName.equals("comboboxentry")) {
					((ComboBoxInputComponentConfig)inputCfg).addElement(atts.getValue("display"), atts.getValue("value"));
				} 
			}
		}

		private InputComponentConfig getInputComponentCfg(Attributes atts) throws SAXException {
			String type=atts.getValue("type");
			String token=atts.getValue("token");
			String desc = atts.getValue("text");
			
			if (type.equals("gallery")){
				GalleryInputComponentConfig galCfg=new GalleryInputComponentConfig(cfg, desc);
				cfg.setGalleryInputComponentConfig(galCfg);
				return galCfg;
			}else{
				if (type.equals("textbox")){
					return new TextBoxInputComponentConfig(token,desc);
				}else if (type.equals("combobox")){
					return new ComboBoxInputComponentConfig(token,desc);
				}else {
					throw new SAXException();
				}
			}
		}
		
		@Override
		public void endElement(String uri, String localName, String qName) throws SAXException {
			if (localName.equals("host")) {
				configs.add(cfg);
				cfg = null;
			}
		}

		@Override
		public void characters(char[] ch, int start, int length) throws SAXException {
			
			
		}

		@Override
		public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
			
			
		}

		@Override
		public void processingInstruction(String target, String data) throws SAXException {
			
			
		}

		@Override
		public void skippedEntity(String name) throws SAXException {
			
			
		}
		
	}
}
