package jbiu.gui;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;

import settings.Settings;
import util.Pair;
import util.exception.CancelException;

import core.io.Application;

import jbiu.init.JBIU;
import jbiu.io.account.AccountManager;
import jbiu.io.login.LoginException;
import jbiu.io.login.LoginManager;
import jbiu.model.LoginObserver;
import jbiu.model.UploaderConfig;

import static jbiu.init.JBIU.rm;

public class GUILoginManager implements LoginObserver{
	
	private List<GUILoginObserver> obs;
	
	private static GUILoginManager instance;
	
	static {
		instance =new GUILoginManager();
	}
	
	public static GUILoginManager getInstance() {
		return instance;
	}
	
	private GUILoginManager(){
		obs=new ArrayList<>();
		LoginManager.setLoginObserver(this);
	}
	
	public void addObserver(GUILoginObserver o) {
		obs.add(o);
	}
	
	
	private Pair<String, char[]> showDialog(String user,char[] pass) throws CancelException, IOException{
		if (user == null||pass==null){
			LoginDialog d=new LoginDialog();
			d.showDialog(user, pass, rm().getLocalizedString("user"), rm().getLocalizedString("pass"));
			return new Pair<String, char[]>(d.getUser(), d.getPass());
		} else {
			return new Pair<String, char[]>(user,pass);
		}
	}
	
	private Pair<String, char[]> showDialogSaveUser(String user,UploaderConfig cfg) throws CancelException, IOException{
		Pair<String, char[]> result=showDialog(user, null);
		AccountManager.setAndSaveLogin(cfg.getInternalName(), result.getValue1(), null);
		return result;
	}
	
	private Pair<String, char[]> showDialogSaveUserAndPass(String user,char[] pass,UploaderConfig cfg) throws CancelException, IOException{
		Pair<String, char[]> result=showDialog(user, pass);
		AccountManager.setAndSaveLogin(cfg.getInternalName(), result.getValue1(), charsToBytes(result.getValue2()));
		return result;
	}
	
	private char[] bytesToChars(byte[] bytes){
		return bytes!=null?Application.UTF8.decode(ByteBuffer.wrap(bytes)).array():null;
	}
	
	private byte[] charsToBytes(char[] chars){
		return chars!=null?Application.UTF8.encode(CharBuffer.wrap(chars)).array():null;
	}
	
	@Override
	public Pair<String, char[]> requestLogin(UploaderConfig cfg) throws CancelException,IOException {
		String user=null;
//		char[] pass=null;
		Settings s=JBIU.getInstance().getSettings();
		Pair<String, char[]> dialogResult;
		if (s.getBoolean("saveUser")) {
			user = AccountManager.getUser(cfg.getInternalName());
			if (s.getBoolean("savePass")){
				dialogResult= showDialogSaveUserAndPass(user, bytesToChars(AccountManager.getPassword(cfg.getInternalName())), cfg);
			} else {
				dialogResult= showDialogSaveUser(user, cfg);
			}
		} else {
			dialogResult=showDialog(null, null);
		}
		/*
		user=dialogResult.getValue1();
		pass=dialogResult.getValue2();
		
		cfg.setHostToken("$HOST#USER", user);
		cfg.setHostToken("$HOST#PASS", new String(pass));
		
		pass=null;
		dialogResult.setValue1(null);
		dialogResult.setValue2(null);
		dialogResult=null;*/
		return dialogResult;
	}
	
	public void updateLoginSuccessful(UploaderConfig cfg) {
		for(GUILoginObserver o:obs)
			o.login(cfg);
	}

	@Override
	public void updateLoginFailed(UploaderConfig cfg, LoginException e) {
		JOptionPane.showMessageDialog(null, rm().getLocalizedString("loginFailed"),"Java Bulk Image Uploader",JOptionPane.ERROR_MESSAGE);
		for(GUILoginObserver o:obs)
			o.loginFailed(cfg);
	}
}
