package jbiu.gui;

import gui.filechooser.FilesChooser;
import gui.resource.RMButton;
import gui.resource.RMLabel;
import gui.util.GuiUtil;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import model.transfer.linkcollector.ILinkCollectorEntry;

import settings.Settings;
import util.exception.CancelException;

import jbiu.init.JBIU;
import jbiu.model.UploadLinkCollector;
import jbiu.model.template.LinkTemplate;

import static jbiu.init.JBIU.rm;

public class LinkPanel extends JPanel {

	private static final long serialVersionUID = 1L;
	
	private DefaultListModel<String> listModel;
	
	private UploadLinkCollector lc;
	private JComboBox<LinkTemplate> cbModifier;
	private int index;
	private JList<String> list;
	
	public void setLinkCollector(UploadLinkCollector lc,int entryIndex) {
		listModel.clear();
		if (lc!=null) {
			Iterator<ILinkCollectorEntry> entries=lc.getEntries();
			
			
			while (entries.hasNext()) {
	
				ILinkCollectorEntry e = entries.next();
				listModel.addElement(e.getLink(entryIndex));
				
			}
		}
		this.lc = lc;
	}
	
	public LinkPanel(UploadLinkCollector lc,int index, String description, List<LinkTemplate> modifiers, LinkTemplate preselectTemplate) {
		this.index=index;
		setLayout(new BorderLayout());
		
		setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(description), BorderFactory.createEmptyBorder(5,5,5,5)));
		final DefaultComboBoxModel<LinkTemplate> cbModifierModel=new DefaultComboBoxModel<>();
		 cbModifier=new JComboBox<>(cbModifierModel);
		
		
		cbModifierModel.addElement(new LinkTemplate(rm().getLocalizedString("none"), null));
		for (LinkTemplate mod:modifiers) {
			cbModifierModel.addElement(mod);
		}
		
		if (preselectTemplate!=null)
			cbModifierModel.setSelectedItem(preselectTemplate);
		
		listModel=new DefaultListModel<>();
		setLinkCollector(lc,index);
		
		list=new JList<>(listModel);
		list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		final JPopupMenu popup=new JPopupMenu();
		
		list.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				GuiUtil.showPopup(popup, e);
			}
			
			@Override
			public void mouseReleased(MouseEvent e) {
				GuiUtil.showPopup(popup, e);
			}
			
			
		});
		
		
		JMenuItem mCopySelected=new JMenuItem(rm().getLocalizedString("copySelected"));
		mCopySelected.addActionListener(new CopySelectedAction());
		JMenuItem mCopyAll=new JMenuItem(rm().getLocalizedString("copy"));
		mCopyAll.addActionListener(new CopyAllAction());
		JMenuItem mSave=new JMenuItem(rm().getLocalizedString("save"));
		mSave.addActionListener(new SaveAction());
		GuiUtil.addAll(popup, mCopySelected,mCopyAll,mSave);
		
		JScrollPane scroll=new JScrollPane(list);
		
		JPanel sidePanel=new JPanel();
		sidePanel.setLayout(new BorderLayout(20,0));
		
//		copyButton=new JButton(rm().getLocalizedString("copy"));
//		saveToFileButton=new JButton(rm().getLocalizedString("saveLinks"));
		
		JButton copyButton=new RMButton(rm(),"copy");
		JButton saveToFileButton=new RMButton(rm(),"save");
		
		copyButton.addActionListener(new CopyAllAction());
		saveToFileButton.addActionListener(new SaveAction());
		sidePanel.add(new RMLabel(rm(), "linkTemplate"),BorderLayout.WEST);
		sidePanel.add(cbModifier,BorderLayout.CENTER);
		
		JPanel pButton=new JPanel(new GridLayout(1, 2));
		
		pButton.add(copyButton);
		pButton.add(saveToFileButton);
		sidePanel.add(pButton,BorderLayout.EAST);
		add(sidePanel,BorderLayout.NORTH);
		add(scroll,BorderLayout.CENTER);
	}
	
	public void clear() {
		lc.clear();
		listModel.clear();
	}
	
	private class CopyAllAction implements ActionListener{
		@Override
		public void actionPerformed(ActionEvent e) {
			
			List<String> links=LinkPanel.this.lc.getLinks(index,GuiUtil.getSelectedValue(cbModifier));
			if (links.size()>0){
				GuiUtil.setClipboardString(links);
			}
		}
	}
	
	private class CopySelectedAction implements ActionListener{

		@Override
		public void actionPerformed(ActionEvent e) {
			int[] selected=list.getSelectedIndices();
			GuiUtil.setClipboardString(LinkPanel.this.lc.getLinks(selected,index,GuiUtil.getSelectedValue(cbModifier)));
		}
	}
	private class SaveAction implements ActionListener{

		@Override
		public void actionPerformed(ActionEvent e) {
			Settings s=JBIU.getInstance().getSettings();
			FilesChooser fc=new FilesChooser(null, rm().getLocalizedString("save"), false, rm().getLocalizedString("ok"), rm().getLocalizedString("cancel"));
			fc.setShowAppendOption(rm().getLocalizedString("append"),s.getBoolean("appendLinks"));
			try {
				fc.showSavePathDialog(s.getPath(JBIU.SETTING_LAST_PATH_SAVE_LINKS));
				LinkPanel.this.lc.save(fc.getSelectedPath(), index,fc.isAppendMode(),GuiUtil.getSelectedValue(cbModifier));
				s.set(JBIU.SETTING_LAST_PATH_SAVE_LINKS, fc.getDir());
				s.set("appendLinks", fc.isAppendMode());
			} catch (CancelException e1) {
				e1.printStackTrace();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
			
		}
	}
	public LinkTemplate getSelectedModifier() {
		return (LinkTemplate)cbModifier.getSelectedItem();
	}
	
	
}
