package jbiu.gui;

import gui.layout.AnchorLayout;
import static gui.layout.AnchorParam.*;
import gui.okcancelpanel.OKCancelPanel;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.border.BevelBorder;

import jbiu.gui.input.AssociatedInputComponent;
import jbiu.gui.input.GalleryInputComponent;
import jbiu.gui.input.InputComponent;
import jbiu.gui.input.model.InputComponentConfig;
import jbiu.model.ULInfo;
import jbiu.model.UploaderConfig;

import util.exception.CancelException;


public class PrepareUploadGUI extends JDialog {

	private static final long serialVersionUID = 1L;
	private InputComponent[][] components;
	private GalleryInputComponent galleryComponent;
	private OKCancelPanel okCancelPanel;
	
	public PrepareUploadGUI(Frame parent) {
		super(parent,true);
	}
	
	public void display(GUIUploaderConfig cfg,ULInfo[] ulInfos) throws IOException, CancelException {
		JPanel cp=(JPanel) getContentPane();

		if (cfg.hasGallery()){
			galleryComponent=(GalleryInputComponent) cfg.getGalleryInputComponentConfig().createInputComponent();
			SwingUtilities.updateComponentTreeUI(galleryComponent);
		}
		
		cp.removeAll();
		cp.setLayout(new BorderLayout());
		setTitle("Prepare upload ["+cfg +"]");
		cp.setBorder(BorderFactory.createEmptyBorder(10, 10, 0, 10));
		cp.add(getInputComps(cfg,ulInfos),BorderLayout.CENTER);
		
		
		okCancelPanel=new OKCancelPanel(this, "Upload", "Cancel");
		cp.add(okCancelPanel,BorderLayout.SOUTH);
		
		setSize(400, 360);
		setVisible(true);
		okCancelPanel.getRetVal();
		getTokensFromGUI(cfg.getUploaderConfig(), ulInfos);
		
	}
	
	private JPanel getRowPanel (int count) {
		return new JPanel(new GridLayout(1, count+1));
	}
	
	private JPanel getInputComps(GUIUploaderConfig cfg,ULInfo[] ulInfos) {
		int count=ulInfos.length;
		JPanel p=new JPanel(new AnchorLayout());
		final int BORDER=10,HGT=24;
		if (galleryComponent!=null) 
			p.add(galleryComponent,AnchorLayout.getConstraints(left(BORDER),top(BORDER),height(24),right(BORDER)));
		
		if (cfg.getInputComponentCount()>0) {
			components=new InputComponent[ulInfos.length][cfg.getInputComponentCount()];
			Iterator<InputComponentConfig> it= cfg.getInputComponentsIterator();
			JPanel view =new JPanel(new AnchorLayout());
			JPanel pRow=getRowPanel(count);
			JPanel pRowDescr=getRowPanel(count);
			pRowDescr.add(new JLabel("Option"));
			pRow=getRowPanel(count);
			pRow.add(new JLabel("Apply to all ->"));
			
			AssociatedInputComponent[] headComps=new AssociatedInputComponent[cfg.getInputComponentCount()];
			
//			JPanel pHead=new JPanel(new AnchorLayout());
//			pHead.setBorder(BorderFactory.createEtchedBorder());
			int k=0;
			int y=0;

			// header (associated) components 
			while (it.hasNext()) {
				AssociatedInputComponent comp=(AssociatedInputComponent) it.next().createAssociatedInputComponent();
				headComps[k++]=comp;
				pRowDescr.add(new JLabel(comp.getDescription()));
				pRow.add(comp);
			}
			view.add(pRowDescr,AnchorLayout.getConstraints(left(BORDER),top(BORDER+HGT*y++),height(HGT),right(BORDER)));
			view.add(pRow,AnchorLayout.getConstraints(left(BORDER),top(BORDER+HGT*y++),height(HGT),right(BORDER)));			
			
			for (int i=0;i<count;i++) {
				k=0;
				pRow=getRowPanel(count);
				JLabel lblFilename=new JLabel(ulInfos[i].getFileName());
				lblFilename.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
				pRow.add(lblFilename);
				it= cfg.getInputComponentsIterator();
				
				while (it.hasNext()) {
					InputComponent comp=(InputComponent) it.next().createInputComponent();
					components[i][k]=comp;	
					comp.setFile(ulInfos[i].getFile());
					headComps[k++].associateInputComponent(comp);
					pRow.add(comp);
				}
				view.add(pRow,AnchorLayout.getConstraints(left(BORDER),top(BORDER+y*HGT),height(HGT),right(BORDER)));
				y++;
			}
			
			
			JScrollPane scroll=new JScrollPane(view);
			scroll.setBorder(BorderFactory.createEtchedBorder());
			p.add(scroll,AnchorLayout.getConstraintsByBorder(BORDER, BORDER, 50, 20));
			view.setPreferredSize(new Dimension(cfg.getInputComponentCount()*100, (count+2)*HGT+BORDER*2));
			
		}
		return p;
	}
	
	/*public void display(GUIUploaderConfig cfg,ULInfo[] ulInfos,UploadManager uploadManager) throws IOException {
		Container cp=getContentPane();
		cp.removeAll();
		cp.setLayout(new BorderLayout());
		setTitle("Prepare upload ["+cfg +"]");
		this.uploadManager=uploadManager;

		okCancelPanel=new OKCancelPanel(this, "Upload", "Cancel");
		cp.add(okCancelPanel,BorderLayout.SOUTH);
		
		if (cfg.getGalleryInputComponent()!=null) {
//			JPanel bTop=new JPanel(new BorderLayout());
//			
			galleryComponent=cfg.getGalleryInputComponent();
//			galleryComponent.setPreferredSize(new Dimension(getWidth()/3, 24));
//			bTop.add(galleryComponent);
			JPanel bTop=new JPanel(new AnchorLayout());
			bTop.setPreferredSize(new Dimension(getWidth(), 50));
			bTop.add(galleryComponent,AnchorLayout.getConstraintsByBorder(50,50,10,10));
			cp.add(bTop,BorderLayout.NORTH);
		}
		
		if (cfg.getInputComponentCount()>0) {
		
			pList=new JPanel(); 
			pList.setLayout(new PercentageLayout());
			pList.setBorder(BorderFactory.createEtchedBorder());
			JScrollPane sp=new JScrollPane(pList);
			sp.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			sp.setBorder(BorderFactory.createEmptyBorder(15,15,15,15));
			cp.add(sp,BorderLayout.CENTER);
			
			setSize(800, 480);
			
		
			
			components=new MultiInputComponent[ulInfos.length][cfg.getInputComponentCount()];
			
			int xstep=98/(cfg.getInputComponentCount()+1);
			int y=0;
			int x=xstep+2;
			Iterator<MultiInputComponent> itComp = cfg.getInputComponentsIterator();
			
			while (itComp.hasNext()) {
				pList.add(new JLabel(itComp.next().getDescription()),getConstraints(x, y, xstep));
				x+=xstep;
			}
			y++;
			x=xstep+2;
			itComp = cfg.getInputComponentsIterator();
			
			int k=0;
			
			MultiInputComponent[] headComps=new MultiInputComponent[cfg.getInputComponentCount()];
			
			
			while (itComp.hasNext()) {
				final MultiInputComponent ic=itComp.next().clone();
				ic.setApplyListener();
				pList.add(ic,getConstraints(x, y, xstep));
				
				x+=xstep;
				
				headComps[k]=ic;
				k++;
			}
			y+=2;
			for (int i=0;i<ulInfos.length;i++) {
				x=2;
				JLabel l=new JLabel(" "+ulInfos[i].getFileName());
				l.setBorder(BorderFactory.createLineBorder(Color.BLACK));
				pList.add(l,getConstraints(x, y, xstep));
				x=xstep+2;
				itComp = cfg.getInputComponentsIterator();
				
				k=0;
				while (itComp.hasNext()) {
					MultiInputComponent comp=itComp.next().clone();
					pList.add(comp,getConstraints(x, y, xstep));
					x+=xstep;
					
					components[i][k]=comp;
					headComps[k].addAssociateInputComponent(comp);
					if (comp instanceof TextInputComponent) {
						((TextInputComponent)comp).setFile(ulInfos[i].getFile());
					}
					
					k++;
				}
				y++;
			}
			pList.setSize(pList.getWidth(), 24*(ulInfos.length+3));
			pList.setPreferredSize(pList.getSize());
		} else if (galleryComponent!=null){
			JPanel bTop=new JPanel();
			setSize(600, 150);
			bTop.setPreferredSize(new Dimension(getWidth(), 50));
			galleryComponent=cfg.getGalleryInputComponent();
			galleryComponent.setPreferredSize(new Dimension(getWidth()/2, 24));
			bTop.add(galleryComponent);
			cp.add(bTop,BorderLayout.CENTER);
			
		}
		if (galleryComponent!=null)
			SwingUtilities.updateComponentTreeUI(galleryComponent);
		setVisible(true);
		okCancelPanel.getRetVal();
		getTokensFromGUI(cfg.getUploaderConfig(), ulInfos);
	}
	
	
	private Object getConstraints(int x,int y,int w) {
		return PercentageLayout.constraints(x, y*24, w, 24, false,true, false, true);
	}*/
	
	private void getTokensFromGUI(UploaderConfig cfg,ULInfo[] ulInfos) throws IOException, CancelException {
		try {
			Map<String, String> tokens;
			if (galleryComponent!=null) 
				tokens=galleryComponent.getMultipleTokens();
			else
				tokens=new HashMap<>();
			
			for (int i=0;i<ulInfos.length;i++) {
			
				ULInfo uinf= ulInfos[i];
				for (String tokenName: tokens.keySet()) {
					uinf.setFileToken(tokenName, new String(tokens.get(tokenName)));
				}
				
				if (components!=null) {
					for (InputComponent comp: components[i]) {
						if (comp.hasMultipleTokens()) {
							tokens=comp.getMultipleTokens();
							
							for (String tokenName: tokens.keySet()) {
								uinf.setFileToken(tokenName, new String(tokens.get(tokenName)));
							}
						} else {
							uinf.setFileToken(comp.getTokenName(),  new String(comp.getTokenValue()));
						}
					}
				}
				
			}
		} finally {
			System.out.println("setvisible");
			setVisible(false);
		}
	}
}
