package jbiu.gui;

import gui.filetree.FileTreeNode;
import gui.settings.SettingsTabbedPane;
import gui.util.lookandfeel.LAFMenuRestart;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JToolBar;

import settings.Settings;
import settings.SettingsLoadable;
import util.exception.CancelException;

import jbiu.gui.settings.SettingsDialog;
import jbiu.init.JBIU;
import jbiu.model.ConfigModel;
import jbiu.model.JBIUModel;
import jbiu.model.UploadManager;

import static jbiu.init.JBIU.*;


public class UploaderGUI extends JFrame implements SettingsLoadable{
	private static final long serialVersionUID = 1L;
	private static final int UPLOAD_TAB_INDEX = 1;

	private SettingsTabbedPane tab;
	private FileChooserPanel fcPanel;
	private UploadManager uploadManager;
	private LAFMenuRestart mLAndF;	
	
	public UploaderGUI() {
		super("Java Bulk Image Uploader v2.0");
		this.uploadManager=JBIUModel.getUploadManager();
		FileTreeNode.setEmpty(JBIU.rm().getLocalizedString(JBIU.LOCALE_EMPTY_NODE));
//		String laf=JBIUModel.getSettings().getString("lookAndFeel");
//		try {
//			GUIConfigLoader2.load(JBIU.getInstance().getConfigurationPath());
//		} catch (IOException e) {
//			throw new RuntimeException(e);
//		}
		GUILoginManager.getInstance();
		Container cp=getContentPane();
		tab=new SettingsTabbedPane("main");
		
		cp.setLayout(new BorderLayout());
		fcPanel=new FileChooserPanel(this, uploadManager);
		tab.addTab("Choose files", fcPanel);
		tab.addTab("Transfers",getUploadPanel() );
		cp.add(tab,BorderLayout.CENTER);
		
		setMenuBar();
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				saveSettings(JBIU.getInstance().getSettings());
				System.exit(0);
			}

			
		});
		loadSettings(JBIU.getInstance().getSettings());

		
		setVisible(true);
		
	}
	

	public void selectUploadTab() {
		tab.setSelectedIndex(UPLOAD_TAB_INDEX);
	}
	
	private JPanel getUploadPanel() {
		JToolBar toolBar = new JToolBar();
		JPanel pUpload=new JPanel();
		pUpload.setLayout(new BorderLayout());
		pUpload.add(toolBar,BorderLayout.NORTH);
		final UploadPanel uploadPanel=new UploadPanel(uploadManager);
		pUpload.add(uploadPanel,BorderLayout.CENTER);
		toolBar.add(new TooltipAbstractAction(rm().getLocalizedString("startStop"), rm().getIcon("startStopIcon")) {
			
			private static final long serialVersionUID = 4462073802775804696L;

			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					uploadManager.startStopAll();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		});
		toolBar.add(new TooltipAbstractAction(rm().getLocalizedString("stopEnqueue"), rm().getIcon("enqueueIcon")) {
			
			private static final long serialVersionUID = 4462073802775804696L;

			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					uploadManager.stopEnqueue(uploadPanel.getSelectedDownloadId());
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		});
		toolBar.add(new TooltipAbstractAction("Link collector", rm().getIcon("iconLinkExtractor")) {
			
			private static final long serialVersionUID = 4330792783853144391L;

			@Override
			public void actionPerformed(ActionEvent e) {
				LinkCollectorGUI.getInstance().showDialog();
			}
		});
		return pUpload;
	}
	
		
	private void setMenuBar() {
		JMenuBar mb=new JMenuBar();
		JMenu mFile=new JMenu(rm().getLocalizedString("menuFile"));
		
		JMenuItem mLogin=new JMenuItem("Login manager");
		mLogin.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
//				AccountManagerGUI loginManager=new AccountManagerGUI(GUIConfigLoader2.getConfigs());
//				loginManager.showDialog();
				showSettings(3);

			}
		});
		JMenuItem mSettings=new JMenuItem("Settings");
		mSettings.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				showSettings(0);
			}
		});
		JMenuItem mUpdate=new JMenuItem("Update");
		mUpdate.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					GUIConfigLoader2.downloadUpdates(false);
					ConfigModel.checkForUpdates();
				} catch (IOException | CancelException e1) {
					e1.printStackTrace();
				}
			}
		});
		
		mb.add(mFile);
		mFile.add(mLogin);
		mFile.add(mSettings);
		mFile.add(mUpdate);
		mLAndF=new LAFMenuRestart(JBIU.getInstance(),JBIU.SETTING_LOOK_AND_FEEL,rm().getLocalizedString("msgLaFRestart"));
		
		JMenu view=new JMenu("View");
		view.add(mLAndF);
		mb.add(view);
		setJMenuBar(mb);
	}
	
	private void showSettings(int listIndex) {
		SettingsDialog sd=new SettingsDialog(UploaderGUI.this);
		try {
			sd.showDialog(listIndex);
			uploadManager.setMaxThreads(JBIU.getInstance().getSettings().getInt(SETTING_THREADS));
			uploadManager.setMaxTries(JBIU.getInstance().getSettings().getInt(SETTING_TRIES));
		} catch (IOException e1) {
			e1.printStackTrace();
		} catch (CancelException e) {
			e.printStackTrace();
		}
	}
	
	private abstract class TooltipAbstractAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		public TooltipAbstractAction(String desc,Icon icon) {
			super(desc, icon);
			putValue(javax.swing.Action.SHORT_DESCRIPTION,desc);
		}
	}

	@Override
	public void loadSettings(Settings s) {
		setBounds(s.getInt(SETTING_X),s.getInt(SETTING_Y),s.getInt(SETTING_WIDTH),s.getInt(SETTING_HEIGHT) );
		tab.loadSettings(s);
		fcPanel.loadSettings(s);
	}

	@Override
	public void saveSettings(Settings settings) {
		try {
			tab.saveSettings(settings);
			settings.set(SETTING_X,getX());
			settings.set(SETTING_Y,getY());
			settings.set(SETTING_WIDTH,getWidth());
			settings.set(SETTING_HEIGHT,getHeight());
			fcPanel.saveSettings(settings);
			JBIU.getInstance().save(uploadManager);
			JBIU.getInstance().saveSettings();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (CancelException e) {
			e.printStackTrace();
		}
	}
}
