package jbiu.gui.input;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;

import util.exception.CancelException;


/**
 *	Header component, other components (for each file) are associated to that component
 */
public abstract class AssociatedInputComponent extends InputComponent {
	private static final long serialVersionUID = 1L;

	protected List<InputComponent> associated;
	protected InputComponent wrappedInputComponent;
	
	public void associateInputComponent(InputComponent ic) {
		if (associated==null ) associated=new LinkedList<InputComponent>();
		associated.add(ic);
	}
	
	protected AssociatedInputComponent(InputComponent inputComponent) {
		super(inputComponent.comp,inputComponent.getInputCfg());
		this.wrappedInputComponent=inputComponent;
		setApplyListener();
	}

	@Override
	public String getTokenValue() {
		return wrappedInputComponent.getTokenValue();
	}

	@Override
	public void setTokenValue(String v) {
		wrappedInputComponent.setTokenValue(v);
	}

	@Override
	public Map<String, String> getMultipleTokens() throws IOException, CancelException {
		return wrappedInputComponent.getMultipleTokens();
	}


	@Override
	public boolean hasMultipleTokens() {
		return wrappedInputComponent.hasMultipleTokens();
	}
	
	public abstract void applyToAssociated() ;
	
	protected abstract void setApplyListener();
	
	protected JButton getApplyButton() {
		JButton bApply=new JButton("Apply");
		bApply.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				applyToAssociated();
			}
		});
		return bApply;
	}
	
}
