package jbiu.gui.input;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CancellationException;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.SwingWorker;

import util.exception.CancelException;

import jbiu.gui.GUIUploaderConfig;
import jbiu.gui.gallery.ExistingGalleryOption;
import jbiu.gui.gallery.GalleryOption;
import jbiu.gui.input.model.GalleryInputComponentConfig;
import jbiu.init.JBIU;
import jbiu.io.gallery.GalleryLoader;
import jbiu.model.UploaderConfig;
import jbiu.model.gallery.ExistingGallery;


public class GalleryInputComponent extends InputComponent {

	private static final long serialVersionUID = 1L;

	private DefaultComboBoxModel<GalleryOption> cbModel;
	private JComboBox<GalleryOption> cbGallery;
	private GUIUploaderConfig cfg;
	
	public GalleryInputComponent(){
		super(null, null);
		init() ;
	}
	
	public GalleryInputComponent(GUIUploaderConfig cfg) {
		super(null, cfg.getGalleryInputComponentConfig());
		init() ;
		setGalleryInputComponent(cfg);
	}
	
	private void init() {
		cbModel=new DefaultComboBoxModel<>();
		cbGallery=new JComboBox<>(cbModel);
		setComp(cbGallery);
		setLoadButton(); 
	}
	
	public void setGalleryInputComponent(GUIUploaderConfig cfg) {
		this.cfg=cfg;
		
		
		cbModel.removeAllElements();
		if (cfg!=null) {
			GalleryInputComponentConfig galleryInputComponent=cfg.getGalleryInputComponentConfig();
			setInputCfg(galleryInputComponent);
			if (galleryInputComponent!=null) {
		
				ExistingGalleryOption.setTokenNames(cfg.getUploaderConfig().getGalleryConfig().getParserTokenNames());
				if (getInputCfg().getNoGalleryOption()!=null) cbModel.addElement(getInputCfg().getNoGalleryOption());
				if (getInputCfg().getNewGalleryOption()!=null) cbModel.addElement(getInputCfg().getNewGalleryOption());
				setEnabled(true);
			} else {
				setEnabled(false);
			}
		}
	}
	
//	private GalleryInputComponent(DefaultComboBoxModel<GalleryOption> model,GalleryInputComponentConfig inputCfg) {
//		super(null, inputCfg);
//		cbGallery=new JComboBox<GalleryOption>();
//		cbGallery.setModel(model);
//		setComp(cbGallery);
//		cbModel=model;
//		setLoadButton();
//	}
	
	
	
	public void addExistingGallery(ExistingGalleryOption galOpt) {
		cbModel.addElement(galOpt);
	}
	
	@Override
	public String getTokenValue() {
		return null;
	}

	@Override
	public Map<String, String> getMultipleTokens() throws IOException, CancelException {
		return ((GalleryOption) cbModel.getSelectedItem()).getTokens(cfg.getUploaderConfig());
	}

	@Override
	public void setTokenValue(String v) {
	}

	@Override
	public boolean hasMultipleTokens() {
		return true;
	}
	
	@Override
	protected GalleryInputComponentConfig getInputCfg() {
		return (GalleryInputComponentConfig) super.getInputCfg();
	}

	protected void afterLoginAction() {
	}
	
	private JButton bLoad;
	
	@Override
	public void setEnabled(boolean enabled) {
		cbGallery.setEnabled(enabled);
		bLoad.setEnabled(enabled);
	}
	
	private void setLoadButton() {
		bLoad =new JButton(JBIU.rm().getLocalizedString("load"));
		add(bLoad,BorderLayout.EAST);
		bLoad.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				bLoad.setEnabled(false);
				new SwingWorker<Void, Void>() {

					@Override
					protected Void doInBackground() throws Exception {
						
						cbGallery.setEnabled(false);
						try {
							cbModel.removeAllElements();
							if (getInputCfg().getNoGalleryOption()!=null) cbModel.addElement(getInputCfg().getNoGalleryOption());
							if (getInputCfg().getNewGalleryOption()!=null) cbModel.addElement(getInputCfg().getNewGalleryOption());
							UploaderConfig cfg=GalleryInputComponent.this.cfg.getUploaderConfig();
							List<ExistingGallery> gals=GalleryLoader.loadExistingGalleries(cfg);
							afterLoginAction();
							boolean firstSet=false;
							
							for (ExistingGallery eg:gals) {
								ExistingGalleryOption opt=new ExistingGalleryOption(eg.getDispName(),eg);
								
								if (!firstSet) {
									cbModel.setSelectedItem(opt);
									firstSet=true;
								}
								
								cbModel.addElement(opt);
								
//								for (InputComponent ic:associated) {
//									((GalleryInputComponent)ic).cbModel.addElement(opt.getDisplay(), opt);
//								}
							}
						} catch (IOException ioex) {
							JOptionPane.showMessageDialog(null, "Loading galleries failed","Java Bulk Image Uploader",JOptionPane.ERROR_MESSAGE);
						} catch (CancellationException ce) {
							System.out.println("cancelled");
						}
						return null;
					}
					
					@Override
					protected void done() {
						bLoad.setEnabled(true);
						cbGallery.setEnabled(true);
					}
				}.execute();
				
			}
		});
		//return bLoad;
	}

//	@Override
//	public void setApplyListener() {
//		JButton bApply=new JButton("Load");
//		add(bApply,BorderLayout.EAST);
//		bApply.addActionListener(new ActionListener() {
//			
//			@Override
//			public void actionPerformed(ActionEvent e) {
//				try {
//					GUILoginManager.login(cfg);
//					List<ExistingGallery> gals=GalleryLoader.loadExistingGalleries(cfg);
//					for (ExistingGallery eg:gals) {
//						GalleryOption opt=new ExistingGalleryOption(eg.getDispName(),cfg.getGalleryConfig().getParserTokenNames(),eg);
//						cbModel.addElement(opt.getDisplay(), opt);
//						
////						for (InputComponent ic:associated) {
////							((GalleryInputComponent)ic).cbModel.addElement(opt.getDisplay(), opt);
////						}
//					}
//				} catch (IOException ioex) {
//					JOptionPane.showMessageDialog(null, "Loading galleries failed");
//				} catch (CancellationException ce) {
//					System.out.println("cancelled");
//				}
//			}
//		});
//		
//		cbGallery.addItemListener(new ItemListener() {
//			
//			@Override
//			public void itemStateChanged(ItemEvent e) {
//				if (e.getStateChange()==ItemEvent.SELECTED) {
////					applyToAssociated();
//				}
//			}
//		});
//	}

//	@Override
//	public JComponent getButtonPanel() {
//		JPanel p=new JPanel();
//		p.setLayout(new GridLayout(1, 3));
//		
//		JButton bNew=new JButton("New");
//		bNew.addActionListener(new ActionListener() {
//			
//			@Override
//			public void actionPerformed(ActionEvent e) {
//				
//			}
//		});
//		
//		JButton bGet=new JButton("Load");
//		bGet.addActionListener(new ActionListener() {
//			
//			@Override
//			public void actionPerformed(ActionEvent e) {
//				try {
//					GUILoginManager.login(cfg);
//					List<ExistingGallery> gals=GalleryLoader.loadExistingGalleries(cfg);
//					for (ExistingGallery eg:gals) {
//						GalleryOption opt=new ExistingGalleryOption(eg.getDispName(),cfg.getGalleryConfig().getParserTokenNames(),eg);
//						cbModel.addElement(opt.getDisplay(), opt);
//						
//						for (InputComponent ic:associated) {
//							((GalleryInputComponent)ic).cbModel.addElement(opt.getDisplay(), opt);
//						}
//					}
//				} catch (IOException ioex) {
//					JOptionPane.showMessageDialog(null, "Loading galleries failed");
//				} catch (CancellationException ce) {
//					System.out.println("cancelled");
//				}
//			}
//		});
//		p.add(bNew);
//		p.add(bGet);
//		p.add(getApplyButton());
//		
//		return p;
//	}

	public void logout(){
		setGalleryInputComponent(this.cfg);
	}
}
