package jbiu.gui.settings;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Collection;
import java.util.List;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;

import jbiu.gui.GUIConfigLoader2;
import jbiu.gui.GUIUploaderConfig;
import jbiu.gui.ReadTosActionListener;
import jbiu.init.JBIU;
import jbiu.model.UploaderConfig;

import settings.Settings;
import core.io.Application;
import gui.layout.percent2.APConstraint;
import gui.layout.percent2.AnchorPercentLayout;
import gui.resource.RMButton;
import gui.resource.ResourceManager;
import gui.settings.dialog.component.AbstractListSettingsComponent;
import gui.settings.dialog.component.CheckboxSettingsComponent;
import gui.settings.dialog.group.SettingsComponentGroup;
import gui.settings.dialog.page.SettingsPage;

public class HostsSettingPage extends SettingsPage{

	private static final long serialVersionUID = -1577481282144654803L;

	private JComboBox<GUIUploaderConfig> cbHosts;
	
	public HostsSettingPage(String entry, Application app) {
		super(entry, app);
	}

	@Override
	protected void createComponentGroups(Settings settings, ResourceManager rm) {
		cbHosts=new JComboBox<>();
		setLayout(new AnchorPercentLayout());
		add(cbHosts,APConstraint.constraint().left(0).right(0).top(0));
		addSettingsComponentGroup(new SettingsComponentGroup("Hosts", rm) {
			
			private static final long serialVersionUID = 5904148881447719204L;

			@Override
			protected void createSettingsComponents(ResourceManager rm) {
				Collection<GUIUploaderConfig> cfgs=GUIConfigLoader2.getConfigs();
				
				setLayout(new AnchorPercentLayout());
				final DefaultComboBoxModel<GUIUploaderConfig> cbHostsModel= (DefaultComboBoxModel<GUIUploaderConfig>) cbHosts.getModel();
				for (GUIUploaderConfig cfg:cfgs){
					cbHostsModel.addElement(cfg);
				}
				final ReadTosActionListener readAl=new ReadTosActionListener();
				final AcceptToSComponent chkAcceptAlways=new AcceptToSComponent();
				
				cbHosts.addItemListener(new ItemListener() {
					
					@Override
					public void itemStateChanged(ItemEvent e) {
						
						if (e.getStateChange()==ItemEvent.SELECTED){
							GUIUploaderConfig cfg=(GUIUploaderConfig) cbHostsModel.getSelectedItem();
							readAl.setCfg(cfg.getUploaderConfig());
							chkAcceptAlways.setUploaderCfg(cfg.getUploaderConfig());
						}
					}
				});
				
				JButton bRead=new RMButton(rm,"readToS");
				bRead.addActionListener(readAl);
				add(bRead,APConstraint.constraint().left(10).right(10).top(5));
				addSettingsComponent(chkAcceptAlways,APConstraint.constraint().left(10).right(10).bottom(5));
				
				UploaderConfig cfg=cbHostsModel.getElementAt(0).getUploaderConfig();
				readAl.setCfg(cfg);
				chkAcceptAlways.setUploaderCfg(cfg);
			}
		},APConstraint.linearV(45, 20));
		
		addSettingsComponentGroup(new MultiHostSettingsGroup(rm),APConstraint.linearV(40, 67));
	}
	
	private class MultiHostSettingsGroup extends SettingsComponentGroup {

		private static final long serialVersionUID = 1310667274801522995L;
		
		public MultiHostSettingsGroup(ResourceManager rm) {
			super("Multi host upload distribution", rm);
		}

		@Override
		protected void createSettingsComponents(ResourceManager rm) {
			final HostListSettingsComponent list = new HostListSettingsComponent("multiHosts");
			setLayout(new AnchorPercentLayout());
			addSettingsComponent(list,APConstraint.constraint().left(10).rightPercent(20).bottom(0).top(0));
			JButton bAdd = new JButton("+");
			JButton bRmv = new JButton("-");
			add(bAdd, APConstraint.constraint().leftPercent(80).top(0).height(25).rightPercent(0));
			add(bRmv, APConstraint.constraint().leftPercent(80).top(25).height(25).rightPercent(0));
			bAdd.addActionListener(new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent arg0) {
					list.addValue((GUIUploaderConfig) cbHosts.getSelectedItem());
				}
			});
		}
		
	}
	
	private class HostListSettingsComponent extends AbstractListSettingsComponent<GUIUploaderConfig> {
		public HostListSettingsComponent(String settingsName) {
			super(settingsName);
		}

		@Override
		public void loadSettings(Settings s) {
			String[] arr = s.getStringArray(settingsName);
			List<GUIUploaderConfig> configs= GUIConfigLoader2.getConfigs();
			for (int i=0;i<arr.length;i++) {
				for(GUIUploaderConfig c:configs) {
					if (c.getInternalName().equals(arr[i])) {
						listValsModel.addElement(c);
						break;
					}
				}
			}
		}
		
		@Override
		public void saveSettings(Settings s) {
			String[] arr = new String[listValsModel.size()];
			for (int i=0;i<listValsModel.size();i++) {
				arr[i] = listValsModel.elementAt(i).getInternalName();
			}
			s.set(settingsName, arr);
		}
	}
	
	private class AcceptToSComponent extends CheckboxSettingsComponent{

		
		
		public AcceptToSComponent() {
			super(null,JBIU.rm().getLocalizedString("alwaysAccept"));
			((JCheckBox) getComponent()).addActionListener(new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent e) {
					saveSettings(settings);					
				}
			});
		}

		public void setUploaderCfg(UploaderConfig cfg) {
			settingsName=cfg.getInternalName()+"_acceptTos";
			loadSettings(settings);
		}
		
		@Override
		public void saveSettings(Settings s) {
			if (isSelected())
				super.saveSettings(s);
		}
	}

}
