package jbiu.io;

import model.transfer.ConfigUpdate;
import net.util.LinkExtractor;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.HashMap;
import java.util.Map;

import jbiu.init.JBIU;
import jbiu.io.gallery.GalleryParser;
import jbiu.io.uploaders.actions.AddRequestPropertyAction;
import jbiu.io.uploaders.actions.ApplyGetAction;
import jbiu.io.uploaders.actions.ApplyMultipartAction;
import jbiu.io.uploaders.actions.ExtractLinksAction;
import jbiu.io.uploaders.actions.ExtractTokenAction;
import jbiu.io.uploaders.actions.PostAction;
import jbiu.io.uploaders.actions.PseudoWriteFileAction;
import jbiu.io.uploaders.actions.SetURLAction;
import jbiu.io.uploaders.actions.UploaderAction;
import jbiu.io.uploaders.actions.WriteFieldAction;
import jbiu.io.uploaders.actions.WriteFileAction;
import jbiu.io.uploaders.actions.condition.Condition;
import jbiu.io.uploaders.actions.condition.ConditionalAction;
import jbiu.io.uploaders.actions.condition.FileTokenValueCondition;
import jbiu.model.GalleryConfig;
import jbiu.model.LinkConfig;
import jbiu.model.LoginInfo;
import jbiu.model.UploaderConfig;
import jbiu.model.token.TokenExtractor;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import plugin.PluginLoader;

import util.exception.CancelException;
import xml.reader.DefaultXMLReader;

public class ConfigLoader2 {
	private static PluginLoader pluginLoader = new PluginLoader();
	private static Path path;
	static {
		path = JBIU.getInstance().getConfigurationPath().resolve("hosts.xml");
	}
	
	public static Map<String,UploaderConfig> load() {
		try {
			CfgLoaderHandler cl=new CfgLoaderHandler();
			if (!Files.exists(path)) {
				try {
					downloadUpdates(true);
				} catch (IOException e) {
					e.printStackTrace();
				} catch (CancelException e) {
					e.printStackTrace();
				}
			}
			DefaultXMLReader.read(path, cl );
			return cl.getCfgs();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	
	
	
	public static synchronized boolean downloadUpdates(boolean force) throws IOException, CancelException{
		//return ConfigUpdate.downloadUpdates("jbiu", "hosts",JBIU.getInstance(), JBIU.getInstance().getConfigurationPath(), force);
		String type = "gallery";
		String name = "jbiu";
		Path cfgDir = JBIU.getInstance().getConfigurationPath();
		Path additionalDir=cfgDir.resolve(type);
		
		boolean b1= ConfigUpdate.downloadUpdates(name, "hosts", JBIU.getInstance(), cfgDir, force);
		boolean b2= ConfigUpdate.downloadNonXMLUpdates(name, type, JBIU.getInstance(),additionalDir , force);
		
		return b1 || b2;	
	}
	
	public static synchronized boolean downloadUpdates() throws IOException, CancelException{
		return downloadUpdates(false);
	}


	private static class CfgLoaderHandler implements ContentHandler {

		private static final String VALUE = "value";
		private Map<String,UploaderConfig> cfgs;
		private LoginInfo li;
		private GalleryConfig gal;
		private PostAction post;
		private UploaderConfig cfg;
		
		public CfgLoaderHandler() {
			cfgs=new HashMap<>();
		}
		
		@Override
		public void setDocumentLocator(Locator locator) {
			
		}

		@Override
		public void startDocument() throws SAXException {
			
		}

		@Override
		public void endDocument() throws SAXException {
						
		}

		@Override
		public void startPrefixMapping(String prefix, String uri) throws SAXException {
			
		}

		@Override
		public void endPrefixMapping(String prefix) throws SAXException {
			
		}

		@Override
		public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
			if (localName.equals("host")){
				String host=atts.getValue("name");
				cfg = new UploaderConfig();
				cfg.setInternalName(host);
				cfg.setTosUrl(atts.getValue("tos"));
				cfgs.put(host, cfg);
			} else if (localName.equals("action")) {
				cfg.addAction(getAction(atts));
			} else if (localName.equals("conditionalaction")) {	
				cfg.addAction(getConditionalAction(atts));
			} else if (localName.equals("link")) {
				String type=atts.getValue("extractorType");
				
				if (type==null) throw new SAXException();
				LinkExtractor extractor = LinkExtractor.getLinkExtractor(type);
				
				cfg.addLinkConfig(new LinkConfig(extractor, getExtractorArgs(extractor, atts)));
			} else if (localName.equals("login")) {
				cfg.setLoginRequired();
				li = cfg.getLoginInfo();
				String passive=atts.getValue("passive");
				if (passive!=null && passive.equals("true")) { 
					li.setPassiveLogin();
				} else {
					li.setURL(atts.getValue("url"));
					li.setLogoutUrl(atts.getValue("logout"));
				}
			} else if (localName.equals("loginparam")) {
				li.addParam(atts.getValue("name"), atts.getValue(VALUE));
			} else if (localName.equals("expectedcookie")) {
				li.setExpectedCookieName(atts.getValue("name"));
			} else if (localName.equals("loginexpectedcode")) {
				li.setLoginExpectCode(Integer.parseInt(atts.getValue(VALUE)));
			} else if (localName.equals("loginresponse")) {
				li.setLoginResponse(atts.getValue(VALUE));
			} else if (localName.equals("gallery")) {
				cfg.setGalleryConfig(new GalleryConfig());
				gal=cfg.getGalleryConfig();
				gal.setListUrl(atts.getValue("url"));
			} else if (localName.equals("parser")) {
				Path f=JBIU.getInstance().getConfigurationPath().resolve("gallery").resolve(cfg.getInternalName()+".jar");
				
				if (Files.exists(f)) {
					gal.setParser((GalleryParser) pluginLoader.load(f, "jbiu.io.gallery."+atts.getValue("class")));						
				}
			} else if (localName.equals("creategallery")) {
				gal.setCreateUrl(atts.getValue("url"));
			} else if (localName.equals("parsertoken")) {
				gal.addParserTokenNames(atts.getValue("name"));
			} else if (localName.equals("createpostparam")) {
				gal.addCreatePostParam(atts.getValue("name"), atts.getValue(VALUE));
			} else if (localName.equals("postparam")) {
				post.addPostParam(atts.getValue("name"), atts.getValue(VALUE));
			} else if (localName.equals("default")) {
				cfg.addDefaultFileToken(atts.getValue("token"), atts.getValue(VALUE));
			} 
		}

		private UploaderAction getConditionalAction(Attributes atts) {
			String type = atts.getValue("type");
			
			if (type.equals("fileToken")) {
				UploaderAction a=getAction(atts);
				Condition cond=new FileTokenValueCondition(atts.getValue("tokenname"), atts.getValue("condition"), atts.getValue("neg").equals("true"));
				ConditionalAction ca= new ConditionalAction(a,cond);
				cond.setParentAction(ca);
				return ca;
			}
			throw new IllegalArgumentException();
		}
		
		private UploaderAction getAction(Attributes atts) {
			String name=atts.getValue("name");
			if (name.equals("setUrl")) {
				return new SetURLAction(atts.getValue(VALUE));
			} else if (name.equals("writeFile")) {
				return new WriteFileAction(atts.getValue("field"), atts.getValue(VALUE));
			} else if (name.equals("writeField")) {
				return new WriteFieldAction(atts.getValue("field"), atts.getValue(VALUE));
			} else if (name.equals("applyMultipart")) {
				return new ApplyMultipartAction();
			} else if (name.equals("extractLinks")) {
				return new ExtractLinksAction();
			} else if (name.equals("extractToken")){
				TokenExtractor extractor=TokenExtractor.getTokenExtractor(atts.getValue("extractorType"));
				return new ExtractTokenAction(atts.getValue("token"),  extractor, getExtractorArgs(extractor, atts));
			} else if (name.equals("applyPost")){	
				post = new PostAction();
				return post;
			} else if (name.equals("requestProperty")){	
				return new AddRequestPropertyAction(atts.getValue("property"), atts.getValue(VALUE));
			} else if (name.equals("applyGet")){	
				return new ApplyGetAction();
			} else if (name.equals("writePseudoFile")) {
				return new PseudoWriteFileAction(atts.getValue("field"), atts.getValue(VALUE));	
			} else{
				throw new IllegalArgumentException("name: "+name);
			}
		}
		
		private String[] getExtractorArgs(LinkExtractor extractor, Attributes atts) {
			String[] argNames = extractor.getArgNames();
			String[] extractorArgs = new String[argNames.length];
			
			for (int i=0;i<argNames.length;i++) {
				extractorArgs[i] = atts.getValue(argNames[i]);
			}
			return extractorArgs;
		}
		
		private String[] getExtractorArgs(TokenExtractor extractor, Attributes atts) {
			String[] argNames = extractor.getArgNames();
			if (argNames==null) return null;
			String[] extractorArgs = new String[argNames.length];
			
			for (int i=0;i<argNames.length;i++) {
				extractorArgs[i] = atts.getValue(argNames[i]);
			}
			return extractorArgs;
		}
		
		
		@Override
		public void endElement(String uri, String localName, String qName) throws SAXException {
			
		}

		@Override
		public void characters(char[] ch, int start, int length) throws SAXException {
			
		}

		@Override
		public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
			
		}

		@Override
		public void processingInstruction(String target, String data) throws SAXException {
			
		}

		@Override
		public void skippedEntity(String name) throws SAXException {
			
		}
		
		public Map<String, UploaderConfig> getCfgs() {
			return cfgs;
		}
		
	}

}
