package jbiu.io.login;

import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import net.util.Proxy;
import jbiu.model.LoginInfo;
import jbiu.model.LoginObserver;
import jbiu.model.TokenManager;
import jbiu.model.UploaderConfig;

import shoshone.httpclient.Cookie;
import shoshone.httpclient.PostParameter;
import shoshone.httpclient.RequestProperty;
import shoshone.httpclient.ShoshoneClient;
import util.Pair;
import util.exception.CancelException;

public class LoginManager {
	
	private static LoginObserver loginObserver;
	
	public static void setLoginObserver(LoginObserver loginObserver) {
		LoginManager.loginObserver = loginObserver;
	}
	
	public static void performLogin(UploaderConfig config,Proxy proxy) throws IOException, CancelException {
		performLogin(new ShoshoneClient(null,proxy), config);
	}
	
	public static void performLogin(ShoshoneClient client,UploaderConfig config) throws IOException, CancelException {
		LoginInfo li=config.getLoginInfo();
		if (li!=null && !li.isPassiveLogin() && !li.isLoggedIn()) {
			Pair<String, char[]> credentials = loginObserver.requestLogin(config);
			config.setHostToken("$HOST#USER", credentials.getValue1());
			config.setHostToken("$HOST#PASS", new String(credentials.getValue2()));
			Iterator<RequestProperty> it=li.getRequestProperties();
			
			if (it!=null) {
				while (it.hasNext()) {
					client.addRequestProperty(it.next());
				}
			}
			
			List<PostParameter> loginParams=new LinkedList<>();
			for (Pair<String,String> p:li.getLoginParams()) {
				loginParams.add(new PostParameter(p.getValue1(), TokenManager.resolve(p.getValue2(),null,config,null)));
			}
			
			client.setUrl(TokenManager.resolveUrl(li.getURL(), config.getTokens()));
			client.applyPostRequest(loginParams);
			Set<Cookie> cookies=client.getCookies();
			if ( (li.getExpectedCode()>-1 && client.getResponseCode()!=li.getExpectedCode())
					|| (li.getLoginResponse()!=null &&	!li.getLoginResponse().equals(client.getResponse()))) {
				
				LoginException e=new LoginException();
				loginObserver.updateLoginFailed(config, e);
				throw e;
			}
			if (li.getExpectedCookieName()==null) {
				li.login(cookies);
			} else {	
				
				for (Cookie c:cookies) {
					if (c.getName().equals(li.getExpectedCookieName())) {
						li.login(cookies);
						break;
					}
				}
			}
			config.setCookies(cookies);
			loginObserver.updateLoginSuccessful(config);
		}
	}
	
	public static void performLogout(List<UploaderConfig> configs) throws CancelException {
		try{
			ShoshoneClient client=new ShoshoneClient();
			
			for (UploaderConfig config:configs) {
				try{
					performLogout(config, client);
				} catch (IOException e){
					e.printStackTrace();
				}
			}
		
		} catch (IOException e){
			e.printStackTrace();
		}
	}
	
	private static void performLogout(UploaderConfig config,ShoshoneClient client) throws IOException, CancelException {
		LoginInfo li=config.getLoginInfo();
		if (li!=null) {
			if (config.getCookies()!=null && li.getLogoutUrl()!=null) {
				client.setAutoReferer();
				client.setFollowRedirects(true);
				client.setUrl(li.getLogoutUrl());
				client.addCookies(config.getCookies());
				client.applyGetRequest();
			}
			li.logout();
			config.setCookies(null);
		}
	}
	
	public static void performLogout(UploaderConfig cfg) throws IOException, CancelException {
		performLogout(cfg, new ShoshoneClient());
	}
}
