package jbiu.model;

import java.io.IOException;
import java.nio.file.Path;

import net.util.Proxy;
import jbiu.io.login.LoginException;
import jbiu.io.login.LoginManager;
import jbiu.io.uploaders.Uploader;

import scheduler.SchedulerRunnable2;
import util.exception.CancelException;
import model.transfer.ScheduledTransferManager;
import model.transfer.Status;
import model.transfer.TransferEngine;
import model.transfer.TransferObserver;
import model.transfer.linkcollector.LinkCollector;
import model.transfer.linkcollector.LinkCollectorEntry;


public class ScheduledUploadManager extends ScheduledTransferManager implements UploadManager {
	
	
	public ScheduledUploadManager(int maxThreads, Proxy proxy ) {
		super(maxThreads, proxy);
	}

	@Override
	protected SchedulerRunnable2 createThread(TransferEngine d) {
		return new UploaderThread((Uploader) d);
	}

	@Override
	public ULInfo createULInfo(Path f,int otherLinksCount) throws IOException {
		LinkCollectorEntry le=new LinkCollectorEntry(otherLinksCount);
		le.setFile(f);
		return new ULInfo(f,le);
	}

	
	protected class UploaderThread extends TransferThread {

		protected LinkCollector lc;
		
		public UploaderThread(TransferEngine uploader) {
			super(uploader);
			this.transferInfo=((Uploader) uploader).getUploadInfo();
			this.engine=uploader;
			setLinkCollector();
		}
		
		protected void setLinkCollector() {
			UploaderConfig cfg=(UploaderConfig)engine.getCfg();
			lc = cfg.getLinkCollector();
			System.out.println(lc);
		}
		
		@Override
		protected boolean isValid() {
			return true;
		}
		
		@Override
		public void runAction() {
			ULInfo ulInfo = (ULInfo)transferInfo;
			//Uploader uploader = (Uploader)engine;
				
			try {
				UploaderConfig cfg=(UploaderConfig) engine.getCfg();
				System.out.println("file tokens " + ulInfo.getFileTokens() );
				System.out.println("host tokens "+ cfg.getTokens() );
				
				LoginInfo li=cfg.getLoginInfo();
				if (li!=null) {
					
					synchronized (li) {
						while (!li.isLoggedIn()) {
							try {
//								loginObserver.requestLogin(cfg);
								LoginManager.performLogin(cfg,TransferEngine.getProxy());
								break;
							} catch (CancelException e) {
								scheduler.stop(id);
								if (!scheduler.isRunning()) stopTimer();
								return;
							} catch (LoginException e) {
								return;
							}
						}
					}
				}
				setTransferStatus(ulInfo, engine, Status.loading); //ulInfo.setStatus(Status.initializing);
				engine.startTransfer();
				if (!stop) {
					ulInfo.updateBytes(ulInfo.getFileSize());
					System.out.println("set status finished");
					filesCounter++;
					
					bytesOfFinishedTransfers+=ulInfo.getFileSize();
					lc.addEntry(ulInfo.getCollectorEntry());
					
					setTransferStatus(ulInfo, engine, Status.finished);//ulInfo.setStatus( Status.finished);
				}
			} catch (CancelException e) {
				System.out.println("Cancel-Exc");
				return;
			} catch (IOException e) {
				e.printStackTrace();
				System.out.println("IOexc");
				if (ulInfo.getStatus()!=Status.stopped ) {
							
					if (maxTries>-1) {
						if (ulInfo.getTries()==maxTries) {
							
							ulInfo.setError(e);
							return;
						}
						ulInfo.nextTry();
					}
				}
			} 
		}

	}
	
	/*@Override
	protected void startScheduler() {
		if (numberOfFiles>0) {
			stopAll=false;
			scheduler.setMaxThreads(maxThreads);
			TransferEngine.setTimeout(timeout);
			
			Iterator<SchedulerRunnable2> it=scheduler.getUnfinished().iterator();
			
			while (it.hasNext()) {
				try{
					UploaderThread thread = (UploaderThread) it.next();
					UploaderConfig cfg=(UploaderConfig) thread.getEngine().getCfg();
					loginObserver.requestLogin(cfg);
					LoginManager.performLogin(cfg);	
					TransferInfo info = thread.getTransferInfo();
					if (info.getStatus()!=Status.finished) {
						info.setLockCancel(false);
						setTransferStatus(info, thread.getEngine(), Status.queued); 
					}
				} catch (LoginException e) {
					loginObserver.updateLoginFailed(e);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			startTimer();
			scheduler.start();
		}
	}
	*/
	@Override
	public void allDone() {
		stopTimer();
		for (TransferObserver o:obs)
			o.updateFinishedAll();
		numberOfFiles=0;
		filesCounter=0;
		
	}

	@Override
	public void addUpload(ULInfo ulInfo,UploaderConfig cfg) throws IOException {
		TransferEngine engine=cfg.getEngine(ulInfo);
		add(engine,createThread(engine));
	}
}
