/*	This program is released under GNU General Public License (see COPYING).
 *  It comes AS IS and without any warranty 
 *  (C) 2010,2011 TCM
 */

package jbiu.model;


import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import util.LongRingBuffer;

import model.transfer.TransferInfo;
import model.transfer.linkcollector.ILinkCollectorEntry;


public class ULInfo extends TransferInfo  {
	protected Path file;
	protected Map<String, String> fileTokens; 
	protected ILinkCollectorEntry collectorEntry;
	protected long fileSize;
	protected LongRingBuffer byteHistory;
	
	public ULInfo(Path file,ILinkCollectorEntry collectorEntry) throws IOException {
		this.file=file;
		fileSize=Files.size(file);
		fileTokens = new HashMap<String, String>();
		this.collectorEntry=collectorEntry;
		collectorEntry.setSize(fileSize);
		byteHistory=new LongRingBuffer(8);
	}

	@Override
	public String toString() {
		return status+ " "+file;
	}


	public Path getFile() {
		return file;
	}

	public String getFileToken(String token) {
		return fileTokens.get(token);
	}
	
	public void setFileToken(String name, String value) {
		fileTokens.put(name, value);
	}

	public String getFileName() {
		return file.getFileName().toString();
	}

	public long getFileSize() {
		return fileSize;
	}
	
	public Map<String, String> getFileTokens() {
		return fileTokens;
	}
	
	
//	public void setLink(int index,String link) {
//		collectorEntry.setLink(index,file, link);
//	}
	
	@Override
	public synchronized void updateBytes(long bytes) {
		super.updateBytes(bytes);
		byteHistory.put(currentBytes-prevBytes);
	}
	
	@Override
	public synchronized long getSpeed() {
		long[] arr=byteHistory.getArr();
		long spd=0L;
		if (arr.length>0) {
			for (int i=0;i<arr.length;i++){
				spd+=arr[i];
			}
			System.out.println(Arrays.toString(arr));
			return spd/arr.length;
		} else {
			return 0L;
		}
	}
	
	public ILinkCollectorEntry getCollectorEntry() {
		return collectorEntry;
	}

	public void setFileTokens(Map<String, String> tokens) {
		if (tokens!=null) {
			
			for (String token:tokens.keySet()) {
				setFileToken(token, tokens.get(token)	);				
			}
		}
		
	}
}
