/*	This program is released under GNU General Public License (see COPYING).
 *  It comes AS IS and without any warranty 
 *  (C) 2010,2011 TCM, Open Source Maniax
 */

package jbiu.model;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import shoshone.httpclient.Cookie;

import model.transfer.EngineConfig;
import model.transfer.TransferEngine;
import model.transfer.TransferInfo;

import jbiu.io.uploaders.actions.UploaderAction;

public class UploaderConfig implements EngineConfig {
	private List<UploaderAction> actions;
	private Map<String, String> hostTokens; 
	private LoginInfo loginInfo;
	private GalleryConfig galleryConfig;
	private LinkConfig viewLinkConfig;
	private List<LinkConfig> otherLinkConfigs;
	private String intName,tosUrl;
	private Map<String, String> defaultFileTokens;
	private UploadLinkCollector lc;
	private Set<Cookie> cookies;
	
	public UploaderConfig() {
		actions = new ArrayList<>();
		hostTokens = new HashMap<>();
	}
	
	public void setTosUrl(String tosUrl) {
		this.tosUrl = tosUrl;
	}
	
	public GalleryConfig getGalleryConfig() {
		return galleryConfig;
	}
	
	public void setGalleryConfig(GalleryConfig galleryConfig) {
		this.galleryConfig = galleryConfig;
	}
	
	public void addAction(UploaderAction action) {
		actions.add(action);
	}
	
	public Iterator<UploaderAction> getActionIterator() {
		ArrayList<UploaderAction> copy=new ArrayList<>(actions.size());

		for (UploaderAction ua: actions) {
			copy.add(ua.clone());
		}
		
		return copy.iterator();
	}
	
	public LoginInfo getLoginInfo() {
		return loginInfo;
	}
	
	public void setLoginRequired() {
		this.loginInfo = new LoginInfo();
	}
	
	public void setHostToken(String name, String value) {
		hostTokens.put(name, value);
	}
	
//	public String getTokenString(String name) {
//		if (name.charAt(0)=='<')
//			return hostTokens.get(name);
//		else
//			return name;
//	}
	
	public Map<String, String> getTokens() {
		return hostTokens;
	}
	
	public int getMaxFileSize() {
		return 0;
	}

	public Object getAllowedFileTypes() {
		return null;
	}

	public String getRulesUrl() {
		return tosUrl;
	}
	
	public int getLinkTypeCount() {
		return (otherLinkConfigs!=null)? otherLinkConfigs.size()+1:1;
	}
	 
	public Iterator<LinkConfig> getOtherLinkConfigs() {
		return otherLinkConfigs.iterator();
	}
	
	public void addLinkConfig(LinkConfig c) {
		if (viewLinkConfig==null) {
			viewLinkConfig=c;
		} else {
			if (otherLinkConfigs==null) otherLinkConfigs=new LinkedList<>();
			otherLinkConfigs.add(c);
		}
	}
	
	public LinkConfig getViewLinkConfig() {
		return viewLinkConfig;
	}
	
	public String getInternalName() {
		return intName;
	}
	
	public void setInternalName(String intName) {
		this.intName = intName;
	}


	public void addDefaultFileToken(String name,String value) {
		if (defaultFileTokens==null) defaultFileTokens=new HashMap<String, String>();
		defaultFileTokens.put(name, value);
	}
	
	public Map<String, String> getDefaultFileTokens() {
		return defaultFileTokens;
	}
	
	@Override
	public String toString() {
		return intName;
	}

	
	public UploadLinkCollector getLinkCollector() {
		if (lc==null) lc=new UploadLinkCollector();
		return lc;
	}
	
	public TransferEngine getEngine(TransferInfo ulInfo) throws IOException {
		return new jbiu.io.uploaders.NewGenericUploader((ULInfo) ulInfo, this);
//		return new jbiu.io.uploaders.SimulatorClassic((ULInfo) ulInfo, this);
	}

	public String getToken(String token) {
		return hostTokens.get(token);
	}
	
	public void setCookies(Set<Cookie> cookies) {
		this.cookies = cookies;
	}

	public Set<Cookie> getCookies() {
		return cookies;
	}

	public String getCookieValue(String name) {
		for(Cookie c:cookies){
			if (c.getName().equals(name)) 
				return c.getValue();
		}
		return null;
	}
}
