/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package model;

import init.JID2;
import io.ConfigLoader3;
import io.IDownloaderConfig;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.file.Path;
import java.nio.file.Paths;

import java.util.Map;


import settings.Settings;
import util.exception.CancelException;

public class JIDModel {
	private static Settings settings;
	
	private static Map<String, IDownloaderConfig> downloaderCfgs;
	//private static String[] delimiters={"\"","'","&quot","]","[",")","<","\u0020","\\n","\r","%",">"};;
	
	private static String[] delimiters;
	
	public static void init(Settings settings) throws IOException, CancelException {
		JIDModel.settings=settings;
		ConfigLoader3.setCfgDir(JID2.getInstance().getConfigurationPath());
		resetConfigs();
		delimiters=ConfigLoader3.loadDelimiters();
	} 

	public static Settings getSettings() {
		return settings;
	}
	
	public static Path getOutput() {
		String out=settings.getString(JID2.SETTING_PATH);
		Path p=Paths.get(out);
		if (!p.isAbsolute()) p=JID2.getInstance().getExecPath().resolve(out);
		return p;
	}
	
	public static DLInfo createDLInfo(String urlString,String subdir,String filename) throws MalformedURLException {
		Path p=getOutput();
		if (subdir!=null) p.resolve(subdir);
		return new DLInfo(urlString, p,filename);
	}
	
	public static DLInfo createDLInfo(String l, String subdir) throws MalformedURLException {
		Path p=getOutput();
		if (subdir!=null) p=p.resolve(subdir);
		return new DLInfo(l, p);
	}
	
	public static DLInfo createDLInfo(String l) throws MalformedURLException {
		return new DLInfo(l,getOutput());
	}

//	public static int getMaxThreads() {
//		return settings.getInt("maxThreads");
//	}
//	
//	public static int getMaxTries() {
//		return settings.getInt("maxTries");
//	}
	
//	public static void save(TransferManager<TransferObserver> dm) throws IOException {
//	}

//	public static List<String> loadSession() throws IOException {
//		LinkedList<String> list=new LinkedList<String>();
//		return list;
//	}
	
	
	public static String[] getDelimiters() {
		return delimiters;
	}
	
	public static String getHost(String urlString) throws MalformedURLException {
		URL url=new URL(urlString);
		String u=url.getHost().toLowerCase();
		
		char dot='.';
		
		int k1=u.lastIndexOf(dot);
		int k2=u.lastIndexOf(dot, k1-1);
		if (k1-k2==3) {
			int k3=u.lastIndexOf(dot, k2-1);
			if (k3>-1)
				return u.substring(k3+1);
			else
				return u.substring(k2+1);
		} else if (k2>-1) {
			return u.substring(k2+1);
		} else {
			return u;
		}
	}

	public static IDownloaderConfig getDownloaderConfigByUrl(String url) throws IOException, CancelException {
		return getDownloaderConfig(getHost(url));
	}


	public static IDownloaderConfig getDownloaderConfig(String host) throws IOException, CancelException {
		return downloaderCfgs.get(host);
	}

	private static void resetConfigs() throws IOException, CancelException {
		downloaderCfgs = ConfigLoader3.load();
		downloaderCfgs.putAll(ConfigLoader3.loadCustom());
//		downloaderCfgs=null;
//		downloaderCfgs=new HashMap<>();
	}
	
	public static boolean downloadUpdates() throws IOException, CancelException{
		boolean b=ConfigLoader3.downloadUpdates();
		if (b)resetConfigs();
		return b;
	}
}
