/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package model;

import io.downloaders.Downloader;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;

import net.util.Proxy;

import scheduler.SchedulerRunnable2;
import util.exception.CancelException;
import model.transfer.EngineConfig;
import model.transfer.ScheduledTransferManager;
import model.transfer.Status;
import model.transfer.TransferEngine;
import model.transfer.TransferObserver;

public class ScheduledDownloadManager extends ScheduledTransferManager implements DownloadManager{

	public ScheduledDownloadManager(int maxThreads, Proxy proxy) {
		super(maxThreads, proxy);
	}

	@Override
	protected SchedulerRunnable2 createThread(TransferEngine d) {
		return new DownloaderThread((Downloader) d);
	}

	protected class DownloaderThread extends TransferThread {

		public DownloaderThread(TransferEngine d) {
			super(d);
		}

		@Override
		protected boolean isValid() {
			return true;
		}
		
		@Override
		protected void runAction() {
			
			DLInfo dlInfo = (DLInfo)transferInfo;
			Downloader d=(Downloader)engine;
			
			try {
				if (!stop) {
					
					setTransferStatus(dlInfo, engine, Status.loading);//dlInfo.setStatus(Status.initializing);
					d.startTransfer();
					Path out = dlInfo.getOutputFile();
					
					if (!stop && isValid()) {
						synchronized (fileCounterLock) {
							filesCounter++;
						}
						
						System.out.println(filesCounter);
						setTransferStatus(dlInfo, engine, Status.finished);
						
						if (dlInfo.getFileSize()>-1)
							bytesOfFinishedTransfers+=dlInfo.getFileSize();
						else
							bytesOfFinishedTransfers+=dlInfo.getCurrentBytes();
					} else {
						dlInfo.reset();
						currentTotalBytes=bytesOfFinishedTransfers;
//						prevTotalBytes=currentTotalBytes;
						if (out!=null) Files.delete(out);
						return;
					}
				}
			} catch (FileNotFoundException e) {
				e.printStackTrace();
				dlInfo.setError(e);	
				return;
			} catch (CancelException e) {
				System.out.println("Cancel-Exc");
				currentTotalBytes=bytesOfFinishedTransfers;
//				prevTotalBytes=currentTotalBytes;
				System.out.println("Test " +currentTotalBytes + " "+bytesOfFinishedTransfers);
				dlInfo.reset();
				try {
					if (dlInfo.getOutputFile()!=null&&dlInfo.getFileName()!=null && Files.exists(dlInfo.getOutputFile()))
						Files.delete(dlInfo.getOutputFile());
				} catch (IOException e1) {
					e1.printStackTrace();
				}
				return;
			} catch (IOException e) {
				e.printStackTrace();
				
				
				if (!stop&& dlInfo.getStatus()!=Status.stopped && maxTries>-1) {
					System.out.println("IOexc " + dlInfo.getTries() + " " +maxTries);		
					if (dlInfo.getTries()==maxTries) {
						scheduler.stop(id);
						dlInfo.setError(e);
						return;
					}
					dlInfo.nextTry();
					
				} else {
					
					try {
						if (dlInfo.getOutputFile()!=null&& Files.exists(dlInfo.getOutputFile()))
							Files.delete(dlInfo.getOutputFile());
					} catch (IOException e1) {
						e1.printStackTrace();
					}
					return;
				}
			
			} catch (Exception ex) {
				ex.printStackTrace();
			}
			
		}

	}

//	@Override
//	public void addDownload(String url) throws IOException {
//		addDownload(JIDModel.createDLInfo(url));
//	}

	@Override
	public void addDownload(DLInfo dlInfo) throws IOException, CancelException {
		EngineConfig cfg=JIDModel.getDownloaderConfigByUrl(dlInfo.getUrl());
		if (cfg!=null) {
			add(cfg.getEngine(dlInfo));
		}else{
			add(new io.downloaders.FileDownloader(dlInfo));
		}
		//add(new  io.downloaders.SimulatorDownloader(dlInfo));
	}

	@Override
	public long getDownloadedBytes() {
		return getTransferredBytes();
	}
	
	@Override
	public void allDone() {
		if (!stopAll && filesCounter==numberOfFiles) {
			stopTimer();
			updateBytes();
			for (TransferObserver o:obs) o.updateFinishedAll();
			numberOfFiles=0;
			filesCounter=0;
			bytesOfFinishedTransfers=0;
			currentTotalBytes=0;
		} 
		
	}

}
