/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package model;

import io.IDownloaderConfig;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import net.util.LinkExtractor;

import shoshone.httpclient.UrlUtil;
import util.StringUtil;
import util.exception.CancelException;

import model.urlconfig.URLConfig;



import static model.util.Util.HTTP_TOKEN;

public class WebsiteParserModel {

	private String[] delimiters;
	
	private static WebsiteParserModel wpm;
	
	
	static {
		try {
			wpm=new WebsiteParserModel();
			System.out.println("loaded");
		} catch (IOException e) {
			e.printStackTrace();
		} 
	}
	
	private WebsiteParserModel() throws IOException {
		load();
	}
	
	public static WebsiteParserModel getWebsiteParserModel() {
		return wpm;
	}

	public Set<String> getLinks(String text, URLConfig cfg) {
		return null;
	}
	
	public Set<String> getLinks(String text) throws IOException, CancelException {
		
		final Set<String> links=new LinkedHashSet<String>();
		
		if (text!=null) {
			int pos = text.indexOf(HTTP_TOKEN);
			while (pos>-1) {
				int end=text.length();
				
				for (String delim:delimiters) {
					int d= text.indexOf(delim, pos);
					if (d>-1) end=Math.min(end, d);
				}
				
				
				if (pos>-1&&end>-1 && end>pos) {
					String l=text.substring(pos, end);
					
					try {
						IDownloaderConfig cfg= JIDModel.getDownloaderConfigByUrl(l);
						if ( cfg!=null &&cfg.validLink(l) ) {
							links.add(l);
						}
					} catch (MalformedURLException e) {
						
					}
					
					
				}
				
				pos = text.indexOf(HTTP_TOKEN,pos+1);
			}
		}
		return links;
	}
	
	
	public Set<String> getLinks(String text,String sourceUrl) throws IOException, CancelException {
		
		final Set<String> links=new LinkedHashSet<String>();
		
		if (text!=null) {
			text = LinkExtractor.preFilterHtml(text);
			for (int i=0;i<delimiters.length;i++){
				text = StringUtil.replaceAll(text, delimiters[i], "'");
			}
			
			int pos=0;
			boolean b=sourceUrl!=null&&sourceUrl.length()>0;
			
			while ((pos = text.indexOf("href", pos+1))>-1){
				int k=pos+6;
				String l = text.substring(k, text.indexOf('\'', k));
				if (l.length()>0){
					if (!l.startsWith(HTTP_TOKEN) &&b){
						l=UrlUtil.getAbsolutePath(l, sourceUrl);
					}
					try {
						IDownloaderConfig cfg= JIDModel.getDownloaderConfigByUrl(l);
						if ( cfg!=null &&cfg.validLink(l) ) {
							links.add(l);
						}
					} catch (MalformedURLException e) {
						
					}
				}
			}
		}
		return links;
	}
	
	public Set<String> getLinks(String text,String sourceUrl, String htmlAttr, List<URLConfig> linkConditions) throws IOException, CancelException {
		final Set<String> links=new LinkedHashSet<String>();
		int pos=0;
		boolean b=sourceUrl!=null&&sourceUrl.length()>0;
		text = LinkExtractor.preFilterHtml(text);
		while ((pos = text.indexOf(htmlAttr, pos+1))>-1){
			int k=pos+6;
			String l = text.substring(k, text.indexOf('\'', k));
			if (l.length()>0){
				if (!l.startsWith(HTTP_TOKEN) &&b){
					l=UrlUtil.getAbsolutePath(l, sourceUrl);
				}
				if (linkConditions!=null){
					boolean result = true;
					for (URLConfig urlConfig : linkConditions) {
						if (!urlConfig.matchesExpression(l)) {
							result = false;
							break;
						}
					}
					if (result)
						links.add(l);
				} else {
					links.add(l);
				}
			}
		}
		return links;
	}
	
	private void load() {
		delimiters=JIDModel.getDelimiters();
	}
}
