package model.transfer.linkcollector;

import java.io.BufferedOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import core.io.Application;

public class LinkCollector {
	
	protected List<ILinkCollectorEntry> entries;
	
	public LinkCollector() {
		entries = new LinkedList<>();
	}
	
	public void addEntry(ILinkCollectorEntry e) {
		entries.add(e);
	}
	
	public Iterator<ILinkCollectorEntry> getEntries() {
		return entries.iterator();
	}
	
	public int getEntryCount() {
		return entries.size();
	}
	
	public void remove(int index) {
		entries.remove(index);
	}
	
	public void clear() {
		entries.clear();
	}
	
	public String getLink(int index,int linkIndex){
		return entries.get(index).getLink(linkIndex);
	}
	
	public List<String> getLinks(int index) {
		List<String> list=new ArrayList<>();

		for (ILinkCollectorEntry e:entries) {
			list.add(e.getLink(index));
		}
		return list;
	}
	
	public void save(Path p,int index,boolean append ) throws IOException {
		if (entries.size()>0) {
			
			StandardOpenOption[] opt=append?new StandardOpenOption[]{StandardOpenOption.APPEND,StandardOpenOption.CREATE,StandardOpenOption.WRITE}:Application.DEFAULT_WRITE_OPTIONS;
			
			BufferedOutputStream out=new BufferedOutputStream( Files.newOutputStream(p,opt));
			Iterator<ILinkCollectorEntry> it=entries.iterator();
			if (append&&Files.size(p)>0) out.write('\n');
			
			while (true){
				out.write(it.next().getLink(index).getBytes(Application.UTF8));
				if (it.hasNext()) {
					out.write('\n');
				} else {
					break;
				}
			}
			out.close();
		}
	}
	
}
